#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_content.component.thesaurus import ICollectionsManager, ITagsManager, IThemesManager
from pyams_content.feature.search import ISearchFolder
from pyams_content.shared.view.interfaces.settings import IViewCollectionsSettings, IViewTagsSettings, \
    IViewThemesSettings
from pyams_content.shared.view.zmi.thesaurus import ViewCollectionsEditForm, ViewTagsEditForm, ViewThemesEditForm
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_thesaurus.zmi.widget import ThesaurusTermsTreeFieldWidget

__docformat__ = 'restructuredtext'

from pyams_content import _


@ajax_form_config(name='tags.html',
                  context=ISearchFolder, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class SearchFolderTagsEditForm(ViewTagsEditForm):
    """Search folder tags settings edit form"""

    title = _("Tags settings")
    legend = _("Search folder tags settings")

    fields = Fields(IViewTagsSettings).select('tags')
    fields['tags'].widget_factory = ThesaurusTermsTreeFieldWidget

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        tags = self.widgets.get('tags')
        if tags is not None:
            tags.label_css_class = 'hidden'
            manager = ITagsManager(self.request.root, None)
            if manager is not None:
                tags.thesaurus_name = manager.thesaurus_name
                tags.extract_name = manager.extract_name


@ajax_form_config(name='themes.html',
                  context=ISearchFolder, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class SearchFolderThemesEditForm(ViewThemesEditForm):
    """Search folder themes settings edit form"""

    title = _("Themes settings")
    legend = _("Search folder themes settings")

    fields = Fields(IViewThemesSettings).select('themes')
    fields['themes'].widget_factory = ThesaurusTermsTreeFieldWidget

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        themes = self.widgets.get('themes')
        if themes is not None:
            themes.label_css_class = 'hidden'
            manager = IThemesManager(self.request.root, None)
            if manager is not None:
                themes.thesaurus_name = manager.thesaurus_name
                themes.extract_name = manager.extract_name


@ajax_form_config(name='collections.html',
                  context=ISearchFolder, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class SearchFolderCollectionsEditForm(ViewCollectionsEditForm):
    """Search folder collections settings edit form"""

    title = _("Collections settings")
    legend = _("Search folder collections settings")

    fields = Fields(IViewCollectionsSettings).select('collections')
    fields['collections'].widget_factory = ThesaurusTermsTreeFieldWidget

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        collections = self.widgets.get('collections')
        if collections is not None:
            collections.label_css_class = 'hidden'
            manager = ICollectionsManager(self.request.root, None)
            if manager is not None:
                collections.thesaurus_name = manager.thesaurus_name
                collections.extract_name = manager.extract_name



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery.portlet.skin.interfaces module

This module defines interfaces of gallery portlet renderers settings.
"""

from zope.interface import Interface

from pyams_skin.schema import BootstrapThumbnailsSelectionField

__docformat__ = 'restructuredtext'

from pyams_content import _


class IGalleryPortletDefaultRendererSettings(Interface):
    """Gallery portlet default renderer settings interface"""

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Thumbnails selection"),
        description=_("Selection used to display images thumbnails"),
        default_width=2,
        change_width=True,
        required=False)

    def get_css_cols(self):
        """Return CSS columns width matching current selection"""


class IGalleryPortletCarouselRendererSettings(Interface):
    """Gallery portlet carousel renderer settings interface"""

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Images selection"),
        description=_("Selection used to display images"),
        default_width=12,
        change_width=False,
        required=False)


class IGalleryPortletRandomImageRendererSettings(Interface):
    """Gallery portlet random image renderer settings interface"""

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Images selection"),
        description=_("Selection used to display images"),
        default_width=2,
        change_width=True,
        required=False)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.fields.interfaces module

This module defines form fields paragraph interface.
"""

__docformat__ = 'restructuredtext'

from pyams_content.component.paragraph import IBaseParagraph
from pyams_content.component.paragraph.interfaces import ParagraphRendererChoice

from pyams_content import _


#
# Form fields paragraphs interfaces
#

FORM_FIELDS_PARAGRAPH_TYPE = 'form-fields'
FORM_FIELDS_PARAGRAPH_NAME = _("Form fields")
FORM_FIELDS_PARAGRAPH_RENDERERS = 'PyAMS_content.paragraph.form-fields.renderers'
FORM_FIELDS_PARAGRAPH_ICON_CLASS = 'fas fa-th-list'


class IFormFieldsParagraph(IBaseParagraph):
    """Form fields paragraph"""

    renderer = ParagraphRendererChoice(
        description=_("Presentation template used for form fields"),
        renderers=FORM_FIELDS_PARAGRAPH_RENDERERS)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from collections import OrderedDict
from enum import Enum

from zope.annotation.interfaces import IAttributeAnnotatable
from zope.container.constraints import contains
from zope.container.interfaces import IContainer
from zope.interface import Attribute, Interface
from zope.schema import Bool, Choice, Int
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_i18n.schema import I18nTextLineField
from pyams_portal.interfaces import IPortletRendererSettings
from pyams_thesaurus.interfaces import THESAURUS_EXTRACTS_VOCABULARY, THESAURUS_NAMES_VOCABULARY
from pyams_thesaurus.interfaces.thesaurus import IThesaurusContextManager

__docformat__ = 'restructuredtext'

from pyams_content import _


FILTER_CONTAINER_ANNOTATION_KEY = 'pyams_content.filters'


#
# Filter sorting
#

class FILTER_SORTING(Enum):
    """Filter modes enumeration"""
    ALPHA = 'alpha_asc'
    ALPHA_DESC = 'alpha_desc'
    COUNT = 'count_asc'
    COUNT_DESC = 'count_desc'
    MANUAL = 'manual'


FILTER_SORTING_LABEL = OrderedDict((
    (FILTER_SORTING.ALPHA.value, _("Alphabetical")),
    (FILTER_SORTING.ALPHA_DESC.value, _("Alphabetical (reversed)")),
    (FILTER_SORTING.COUNT.value, _("Results count")),
    (FILTER_SORTING.COUNT_DESC.value, _("Results count (reversed)")),
    (FILTER_SORTING.MANUAL.value, _("Manual"))
))

FILTER_SORTING_VOCABULARY = SimpleVocabulary([
    SimpleTerm(k, title=v)
    for k, v in FILTER_SORTING_LABEL.items()
])


MANUAL_FILTER_SORTING_LABEL = FILTER_SORTING_LABEL.copy()
MANUAL_FILTER_SORTING_LABEL['manual'] = _("Manual")

MANUAL_FILTER_SORTING_VOCABULARY = SimpleVocabulary([
    SimpleTerm(k, title=v)
    for k, v in MANUAL_FILTER_SORTING_LABEL.items()
])


#
# Filter dispay mode
#

class FILTER_DISPLAY_MODE(Enum):
    """Filter display modes enumeration"""
    LIST = 'list'
    SELECT = 'select'


FILTER_DISPLAY_MODE_LABEL = (
    (FILTER_DISPLAY_MODE.LIST.value, _("List")),
    (FILTER_DISPLAY_MODE.SELECT.value, _("Choice")),
)

FILTER_DISPLAY_MODE_VOCABULARY = SimpleVocabulary([
    SimpleTerm(key, title=value)
    for key, value in FILTER_DISPLAY_MODE_LABEL
])


#
# Content-type filter mode
#

class CONTENT_TYPE_FILTER_MODE(Enum):
    """Content-type filter modes enumeration"""
    FACET_LABEL = 'facet_label'
    FACET_TYPE_LABEL = 'facet_type_label'


CONTENT_TYPE_FILTER_MODE_LABEL = {
    CONTENT_TYPE_FILTER_MODE.FACET_LABEL.value: _("Content-type label"),
    CONTENT_TYPE_FILTER_MODE.FACET_TYPE_LABEL.value: _("Data type label")
}

CONTENT_TYPE_FILTER_MODE_VOCABULARY = SimpleVocabulary([
    SimpleTerm(key, title=value)
    for key, value in CONTENT_TYPE_FILTER_MODE_LABEL.items()
])


#
# Filters interfaces
#

class IFilterValues(Interface):
    """Filter value interface"""


class IFilterIndexInfo(Interface):
    """Filter index info base interface"""

    facets = Attribute("Filter facets")


class IFilter(Interface):
    """Base filter interface"""

    visible = Bool(title=_("Visible?"),
                   description=_("Is this filter visible in front-office?"),
                   required=True,
                   default=True)

    label = I18nTextLineField(title=_("Label"),
                              description=_("This is the label displayed in filter header"),
                              required=True)

    display_mode = Choice(title=_("Display mode"),
                          description=_("Filter entries display mode"),
                          vocabulary=FILTER_DISPLAY_MODE_VOCABULARY,
                          default=FILTER_DISPLAY_MODE.LIST.value,
                          required=True)

    open_state = Bool(title=_("Default open state"),
                      description=_("If 'no', options list will be collapsed by default"),
                      required=True,
                      default=True)

    displayed_entries = Int(title=_("Displayed entries"),
                            description=_("Number of entries displayed in search filter"),
                            required=True,
                            default=5)

    display_count = Bool(title=_("Display results count"),
                         description=_("In 'list' mode, display number of entries matching "
                                       "each search filter value"),
                         required=True,
                         default=True)

    select_placeholder = I18nTextLineField(title=_("Select placeholder"),
                                           description=_("Placeholder text displayed in the "
                                                         "select widget"),
                                           required=False)

    sorting_mode = Choice(title=_("Sorting mode"),
                          description=_("Filter entries sorting mode"),
                          vocabulary=FILTER_SORTING_VOCABULARY,
                          default=FILTER_SORTING.ALPHA.value,
                          required=True)

    filter_type = Attribute("Filter type")

    def is_visible(self, request=None):
        """Is association item published?"""


class IContentTypesFilter(IFilter):
    """Content-types filter interface"""

    content_mode = Choice(title=_("Content-type display mode"),
                          description=_("Filter entries display mode"),
                          vocabulary=CONTENT_TYPE_FILTER_MODE_VOCABULARY,
                          default=CONTENT_TYPE_FILTER_MODE.FACET_LABEL.value,
                          required=True)


class ITitleFilter(IFilter):
    """Title filter"""


class IThesaurusFilter(IFilter, IThesaurusContextManager):
    """Thesaurus-based filter interface"""

    thesaurus_name = Choice(title=_("Thesaurus name"),
                            description=_("Name of thesaurus used to get filter terms"),
                            vocabulary=THESAURUS_NAMES_VOCABULARY,
                            required=True)

    extract_name = Choice(title=_("Thesaurus extract"),
                          description=_("Name of thesaurus extract containing terms used by this filter"),
                          vocabulary=THESAURUS_EXTRACTS_VOCABULARY,
                          required=False)


class ITagsFilter(IThesaurusFilter):
    """Tags filter Interface"""


class IThemesFilter(IThesaurusFilter):
    """Themes filter Interface"""


class ICollectionsFilter(IThesaurusFilter):
    """Collections filter Interface"""


class IFilterType(Interface):
    """Filter type interface"""


class IFiltersContainer(IContainer):
    """Filters container interface"""

    contains(IFilter)

    def add(self, obj):
        """Add filter to container"""

    def get_visible_filters(self):
        """Visible filters iterator"""

    def get_processed_filters(self, context, request, aggregations):
        """Iterator over processed filters aggregations"""


class IFiltersContainerTarget(IAttributeAnnotatable):
    """Filters container target marker interface"""


class IAggregatedPortletRendererSettings(IPortletRendererSettings):
    """Aggregated portlet renderer settings marker interface"""


class IFilterAggregate(Interface):
    """Filter aggregate getter"""


class IFilterProcessor(Interface):
    """Filter processor interface"""

    def process(self, aggregations, filter_type=None):
        """Process the filter and return relevant data"""

    def get_aggregations(self, aggregations):
        """Return aggregations from search results"""


class IFilterProcessorAggregationsHandler(Interface):
    """Filter processor aggregations handler interface"""

    def get_aggregations(self, aggregations):
        """Convert search aggregations in renderer format"""



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph.html module

This module defines raw and rich text HTML paragraphs.
"""

__docformat__ = 'restructuredtext'

from pyramid.events import subscriber
from zope.interface import implementer
from zope.lifecycleevent import IObjectAddedEvent, IObjectModifiedEvent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.extfile.interfaces import IExtFileContainerTarget
from pyams_content.component.illustration import IIllustrationTarget
from pyams_content.component.links.html import check_content_links
from pyams_content.component.links.interfaces import ILinkContainerTarget
from pyams_content.component.paragraph import BaseParagraph
from pyams_content.component.paragraph.interfaces import IBaseParagraph
from pyams_content.component.paragraph.interfaces.html import HTML_PARAGRAPH_ICON_CLASS, \
    HTML_PARAGRAPH_NAME, HTML_PARAGRAPH_RENDERERS, HTML_PARAGRAPH_TYPE, IHTMLParagraph, \
    IRawParagraph, RAW_PARAGRAPH_ICON_CLASS, RAW_PARAGRAPH_NAME, RAW_PARAGRAPH_RENDERERS, \
    RAW_PARAGRAPH_TYPE
from pyams_content.feature.renderer import RenderersVocabulary
from pyams_utils.factory import factory_config
from pyams_utils.vocabulary import vocabulary_config


#
# Raw HTML code paragraph
#

@factory_config(IRawParagraph)
@factory_config(IBaseParagraph, name=RAW_PARAGRAPH_TYPE)
class RawParagraph(BaseParagraph):
    """Raw paragraph persistent class"""

    factory_name = RAW_PARAGRAPH_TYPE
    factory_label = RAW_PARAGRAPH_NAME
    factory_intf = IRawParagraph

    secondary = True

    icon_class = RAW_PARAGRAPH_ICON_CLASS

    body = FieldProperty(IRawParagraph['body'])
    renderer = FieldProperty(IRawParagraph['renderer'])


@vocabulary_config(name=RAW_PARAGRAPH_RENDERERS)
class RawParagraphRenderersVocabulary(RenderersVocabulary):
    """Raw paragraph renderers vocabulary"""

    content_interface = IRawParagraph


#
# Rich text HTML paragraph
#

@factory_config(IHTMLParagraph)
@factory_config(IBaseParagraph, name=HTML_PARAGRAPH_TYPE)
@implementer(IIllustrationTarget, IExtFileContainerTarget, ILinkContainerTarget)
class HTMLParagraph(BaseParagraph):
    """HTML paragraph persistent class"""

    factory_name = HTML_PARAGRAPH_TYPE
    factory_label = HTML_PARAGRAPH_NAME
    factory_intf = IHTMLParagraph

    icon_class = HTML_PARAGRAPH_ICON_CLASS

    body = FieldProperty(IHTMLParagraph['body'])
    renderer = FieldProperty(IHTMLParagraph['renderer'])


@subscriber(IObjectAddedEvent, context_selector=IHTMLParagraph)
@subscriber(IObjectModifiedEvent, context_selector=IHTMLParagraph)
def handle_html_paragraph_links(event):
    """Handle HTML paragraph links"""
    paragraph = event.object
    for lang, body in (paragraph.body or {}).items():
        check_content_links(paragraph, body, lang, notify=False)


@vocabulary_config(name=HTML_PARAGRAPH_RENDERERS)
class HTMLParagraphRenderersVocabulary(RenderersVocabulary):
    """HTML paragraph renderers vocabulary"""

    content_interface = IHTMLParagraph



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi.types module

This module provides shared content data types management components.
"""

from zope.interface import Interface

from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_content.reference.pictogram.zmi.widget import PictogramSelectFieldWidget
from pyams_content.shared.common.interfaces.types import IDataType, ITypedDataManager, ITypedSharedTool
from pyams_content.shared.common.zmi.types.interfaces import ISharedToolTypesTable
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.group import Group
from pyams_form.interfaces.form import IAJAXFormRenderer, IGroup
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_skin.interfaces.view import IModalEditForm
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces.intids import IUniqueID
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm, FormGroupSwitcher
from pyams_zmi.helper.event import get_json_table_row_add_callback, get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, IObjectHint, IObjectLabel, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IToolbarViewletManager
from pyams_zmi.table import TableElementEditor
from pyams_zmi.utils import get_object_hint, get_object_label


__docformat__ = 'restructuredtext'

from pyams_content import _


#
# Shared data types views
#

@viewlet_config(name='add-data-type.action',
                context=ITypedSharedTool, layer=IAdminLayer, view=ISharedToolTypesTable,
                manager=IToolbarViewletManager, weight=20,
                permission=MANAGE_TOOL_PERMISSION)
class DataTypesAddAction(ContextAddAction):
    """Data type add action"""

    label = _("Add content type")
    href = 'add-data-type.html'


@ajax_form_config(name='add-data-type.html',
                  context=ITypedSharedTool, layer=IPyAMSLayer,
                  permission=MANAGE_TOOL_PERMISSION)
class DataTypeAddForm(AdminModalAddForm):
    """Data type add form"""

    subtitle = _("New content type")
    legend = _("New data type properties")

    @property
    def fields(self):
        """Form fields getter"""
        fields = Fields(IDataType).select('label', 'source_folder',
                                          'color', 'pictogram', 'field_names')
        fields['pictogram'].widget_factory = PictogramSelectFieldWidget
        if not self.context.shared_content_info_factory:
            fields = fields.omit('field_names')
        return fields

    content_factory = IDataType

    def add(self, obj):
        oid = IUniqueID(obj).oid
        ITypedDataManager(self.context)[oid] = obj


@adapter_config(required=(ITypedSharedTool, IAdminLayer, DataTypeAddForm),
                provides=IAJAXFormRenderer)
class DataTypeAddFormRenderer(ContextRequestViewAdapter):
    """Data type add form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        return {
            'status': 'success',
            'callbacks': [
                get_json_table_row_add_callback(self.context, self.request,
                                                ISharedToolTypesTable, changes)
            ]
        }


@adapter_config(required=(IDataType, IAdminLayer, Interface),
                provides=IObjectLabel)
def data_type_label(context, request, view):
    """Data type label"""
    i18n = II18n(context)
    return i18n.query_attribute('label', request=request)


@adapter_config(required=(IDataType, IAdminLayer, Interface),
                provides=IObjectHint)
def data_type_hint(context, request, view):  # pylint: disable=unused-argument
    """Data type hint"""
    return request.localizer.translate(_("Data type"))


@adapter_config(required=(IDataType, IAdminLayer, ISharedToolTypesTable),
                provides=ITableElementEditor)
class DataTypeEditor(TableElementEditor):
    """Data type editor"""


@ajax_form_config(name='properties.html',
                  context=IDataType, layer=IPyAMSLayer,
                  permission=MANAGE_TOOL_PERMISSION)
class DataTypeEditForm(AdminModalEditForm):
    """Data type properties edit form"""

    legend = _("Content type properties")

    @property
    def fields(self):
        """Form fields getter"""
        fields = Fields(IDataType).select('label', 'source_folder',
                                          'color', 'pictogram', 'field_names')
        fields['pictogram'].widget_factory = PictogramSelectFieldWidget
        tool = get_parent(self.context, ITypedSharedTool)
        if (tool is not None) and not tool.shared_content_info_factory:
            fields = fields.omit('field_names')
        return fields


@adapter_config(required=(IDataType, IAdminLayer, IModalEditForm),
                provides=IFormTitle)
def data_type_edit_form_title(context, request, form):
    """Data type edit form title getter"""
    return TITLE_SPAN_BREAK.format(
        get_object_hint(context, request, form),
        get_object_label(context, request, form))


@adapter_config(required=(IDataType, IAdminLayer, DataTypeEditForm),
                provides=IAJAXFormRenderer)
class DataTypeEditFormRenderer(ContextRequestViewAdapter):
    """Data type edit form AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        tool = get_parent(self.context, ITypedSharedTool)
        return {
            'callbacks': [
                get_json_table_row_refresh_callback(tool, self.request,
                                                    ISharedToolTypesTable, self.context)
            ]
        }


@adapter_config(name='labels.group',
                required=(ITypedSharedTool, IAdminLayer, DataTypeAddForm),
                provides=IGroup)
@adapter_config(name='labels.group',
                required=(IDataType, IAdminLayer, DataTypeEditForm),
                provides=IGroup)
class DataTypeLabelsGroup(Group):
    """Data type labels group"""

    legend = _("Contents labels")
    weight = 10

    fields = Fields(IDataType).select('display_as_tag', 'navigation_label', 'facets_label',
                                      'facets_type_label', 'dashboard_label')


@adapter_config(name='pictograms.group',
                required=(ITypedSharedTool, IAdminLayer, DataTypeAddForm),
                provides=IGroup)
@adapter_config(name='pictograms.group',
                required=(IDataType, IAdminLayer, DataTypeEditForm),
                provides=IGroup)
class DataTypePictogramsGroup(FormGroupSwitcher):
    """Data type pictograms group"""

    legend = _("Pictograms")
    weight = 20

    fields = Fields(IDataType).select('pictogram_on', 'pictogram_off')
    fields['pictogram_on'].widget_factory = PictogramSelectFieldWidget
    fields['pictogram_off'].widget_factory = PictogramSelectFieldWidget



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.fields.handler.mail module

This module defines a custom form handler used to send emails.
"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.fields.handler.interfaces import IMailtoHandlerInfo, IMailtoHandlerTarget, \
    MAILTO_HANDLER_ANNOTATION_KEY
from pyams_fields.interfaces import IFormFieldContainer, IFormHandler, IFormHandlerInfo, IFormHandlersInfo
from pyams_i18n.interfaces import II18n
from pyams_mail.message import HTMLMessage
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.notification import INotificationSettings
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import MISSING_INFO
from pyams_utils.list import boolean_iter
from pyams_utils.registry import get_utility, utility_config
from pyams_utils.request import query_request


__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(IMailtoHandlerInfo)
class MailtoHandlerInfo(Persistent, Contained):
    """Mailto handler persistent class"""

    service_name = FieldProperty(IMailtoHandlerInfo['service_name'])
    source_name = FieldProperty(IMailtoHandlerInfo['source_name'])
    source_address = FieldProperty(IMailtoHandlerInfo['source_address'])
    target_name = FieldProperty(IMailtoHandlerInfo['target_name'])
    target_address = FieldProperty(IMailtoHandlerInfo['target_address'])
    notification_message = FieldProperty(IMailtoHandlerInfo['notification_message'])
    confirm_message = FieldProperty(IMailtoHandlerInfo['confirm_message'])


@adapter_config(required=IMailtoHandlerTarget,
                provides=IMailtoHandlerInfo)
@adapter_config(name='mailto',
                required=IMailtoHandlerTarget,
                provides=IFormHandlerInfo)
def mailto_handler_info(context):
    """Mailto form handler info"""
    return get_annotation_adapter(context, MAILTO_HANDLER_ANNOTATION_KEY,
                                  IMailtoHandlerInfo,
                                  name='++handler++mailto')


@utility_config(name='mailto',
                provides=IFormHandler)
class SimpleMailtoFormHandler:
    """Simple mail form handler"""

    label = _("Simple mail handler")
    weight = 10

    target_interface = IMailtoHandlerTarget

    @staticmethod
    def build_message(template, data):
        """Message builder"""
        if template:
            return template.format(**data)
        return '<br />'.join((
            f"{k}: {v or MISSING_INFO}"
            for k, v in data.items()
        ))

    def handle(self, form, data, user_data):
        """Form data handler"""
        sm = get_utility(ISecurityManager)
        settings = INotificationSettings(sm)
        if not settings.enable_notifications:
            return
        mailer = settings.get_mailer()
        request = query_request()
        handlers = IFormHandlersInfo(form, None)
        if handlers is None:
            return
        handler_info = IMailtoHandlerInfo(handlers, None)
        if handler_info is None:
            return
        i18n_handler = II18n(handler_info)
        service_name = i18n_handler.query_attribute('service_name', request=request)
        if handler_info.target_address:
            body = self.build_message(
                i18n_handler.query_attribute('notification_message', request=request),
                user_data)
            target = f'{handler_info.target_name} <{handler_info.target_address}>' \
                if handler_info.target_name else handler_info.target_address
            message = HTMLMessage(
                subject=f'[{settings.subject_prefix}] {service_name}',
                from_addr=f'{handler_info.source_name} <{handler_info.source_address}>',
                to_addr=target,
                html=body)
            mailer.send(message)
        fields = IFormFieldContainer(form, None)
        if fields is None:
            return
        has_email, email_field = boolean_iter(fields.find_fields('mail'))
        if not has_email:
            return
        email_address = data.get(next(email_field).name)
        confirm_message = i18n_handler.query_attribute('confirm_message', request=request)
        if not (email_address and confirm_message):
            return
        message = HTMLMessage(
            subject=f'[{settings.subject_prefix}] {service_name}',
            from_addr=f'{handler_info.source_name} <{handler_info.source_address}>',
            to_addr=email_address,
            html=confirm_message.format(**user_data))
        mailer.send(message)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.skin.metas module

This module defines meta-headers which are common to all shared contents.
"""

from zope.dublincore.interfaces import IZopeDublinCore
from zope.interface import Interface

from pyams_content.component.illustration import IIllustration, IIllustrationTarget, ILinkIllustration
from pyams_content.component.thesaurus import ITagsInfo
from pyams_content.root import ISiteRootInfos
from pyams_content.shared.common import IWfSharedContent
from pyams_file.interfaces.thumbnail import IThumbnails
from pyams_i18n.interfaces import II18n, II18nManager, INegotiator
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_skin.interfaces.metas import IHTMLContentMetas
from pyams_skin.metas import ContentMeta, PropertyMeta, SchemaMeta
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.registry import get_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url, canonical_url
from pyams_workflow.interfaces import IWorkflowPublicationInfo

__docformat__ = 'restructuredtext'


@adapter_config(name='opengraph',
                required=(IWfSharedContent, IPyAMSUserLayer, Interface),
                provides=IHTMLContentMetas)
class SharedContentOpengraphMetasAdapter(ContextRequestViewAdapter):
    """Shared content opengraph metas adapter"""

    weight = 15

    def get_metas(self):
        context = self.context
        request = self.request
        i18n = II18n(context)
        negotiator = get_utility(INegotiator)
        lang = negotiator.server_language

        description = i18n.query_attribute('description', lang=lang, request=request) or \
            i18n.query_attribute('header', lang=lang, request=request)

        # main properties
        yield PropertyMeta('og:type', 'article')
        yield PropertyMeta('og:title', i18n.query_attribute('title', lang=lang, request=request))
        if description:
            yield PropertyMeta('og:description', description)

        # URL and site name
        yield PropertyMeta('og:url', canonical_url(context, request))
        configuration = ISiteRootInfos(request.root)
        yield PropertyMeta('og:site_name', configuration.title)

        # workflow information
        dc = IZopeDublinCore(context, None)
        if (dc is not None) and dc.modified:
            yield PropertyMeta('article:modified_time', tztime(dc.modified).isoformat())
        pub_info = IWorkflowPublicationInfo(context, None)
        if pub_info is not None:
            if pub_info.first_publication_date:
                yield PropertyMeta('article:published_time',
                                   tztime(pub_info.first_publication_date).isoformat())
            if pub_info.publication_expiration_date:
                yield PropertyMeta('article:expiration_time',
                                   tztime(pub_info.publication_expiration_date).isoformat())

        # tags
        tags = ITagsInfo(context, None)
        if tags is not None:
            for tag in tags.tags or ():
                yield PropertyMeta('article:tag', tag.label)

        # illustration properties
        illustration = None
        card = None
        card_url = None
        alt = None
        target = context
        while target is not None:
            illustration = ILinkIllustration(target, None)
            if (illustration is None) or (not illustration.has_data()):
                illustration = IIllustration(target, None)
            if (illustration is not None) and illustration.has_data():
                break
            target = get_parent(target, IIllustrationTarget, allow_context=False)
        if (target is not None) and (illustration is not None):
            data = II18n(illustration).query_attribute('data', lang=lang, request=request)
            if data:
                card = IThumbnails(data).get_thumbnail('card:w800')
                card_url = absolute_url(data, request, '++thumb++card:w800')
                yield PropertyMeta('og:image', card_url)
                if request.scheme == 'https':
                    yield PropertyMeta('og:image:secure_url', card_url)
                else:
                    yield PropertyMeta('og:image:url', card_url)
                yield PropertyMeta('og:image:type', card.content_type)
                image_size = card.image_size
                yield PropertyMeta('og:image:width', image_size[0])
                yield PropertyMeta('og:image:height', image_size[1])
                alt = II18n(illustration).query_attribute('alt_title', lang=lang, request=request)
                if alt:
                    yield PropertyMeta('og:image:alt', alt)

        # locales properties
        yield PropertyMeta('og:locale', lang)
        manager = II18nManager(context, None)
        if manager is not None:
            for other_lang in manager.languages or ():
                if other_lang != lang:
                    yield PropertyMeta('og:locale:alternate', other_lang)

        yield ContentMeta('twitter:title',
                          i18n.query_attribute('title', lang=lang, request=request))
        if description:
            yield ContentMeta('twitter:description', description)
        if card is not None:
            yield ContentMeta('twitter:card', 'summary_large_image')
            yield ContentMeta('twitter:image', card_url)
            if alt:
                yield ContentMeta('twitter:image:alt', alt)
        else:
            yield ContentMeta('twitter:card', 'summary')

        # Schema.org properties
        yield SchemaMeta('name', i18n.query_attribute('title', lang=lang, request=request))
        if description:
            yield SchemaMeta('description', description)
        if card is not None:
            yield SchemaMeta('image', card_url)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.root module

This module defines roles and permissions checker of site root.
"""

from persistent import Persistent
from pyramid.events import subscriber
from zope.container.contained import Contained
from zope.interface import Interface, implementer
from zope.schema.fieldproperty import FieldProperty
from zope.traversing.interfaces import ITraversable

from pyams_content.interfaces import MANAGE_SITE_ROOT_PERMISSION
from pyams_content.root.interfaces import ISiteRootInfos, ISiteRootRoles, \
    ISiteRootToolsConfiguration, SITEROOT_ROLES, SITE_ROOT_INFOS_KEY, \
    SITE_ROOT_TOOLS_CONFIGURATION_KEY
from pyams_file.property import FileProperty
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces import IGrantedRoleEvent, IRolesPolicy, ISecurityManager, IViewContextPermissionChecker
from pyams_security.interfaces.base import IRole
from pyams_security.interfaces.plugin import ILocalGroup
from pyams_security.property import RolePrincipalsFieldProperty
from pyams_security.security import ProtectedObjectRoles
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import ContextAdapter, ContextRequestViewAdapter, adapter_config, \
    get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.list import next_from
from pyams_utils.registry import get_utility

__docformat__ = 'restructuredtext'

from pyams_utils.traversing import get_parent


@factory_config(provided=ISiteRootInfos)
class SiteRootInfos(Persistent, Contained):
    """Site root information"""

    title = FieldProperty(ISiteRootInfos['title'])
    short_title = FieldProperty(ISiteRootInfos['short_title'])
    description = FieldProperty(ISiteRootInfos['description'])
    author = FieldProperty(ISiteRootInfos['author'])
    icon = FileProperty(ISiteRootInfos['icon'])
    logo = FileProperty(ISiteRootInfos['logo'])
    public_url = FieldProperty(ISiteRootInfos['public_url'])
    support_email = FieldProperty(ISiteRootInfos['support_email'])


@adapter_config(required=ISiteRoot, provides=ISiteRootInfos)
def site_root_infos_factory(context):
    """Site root information factory"""
    return get_annotation_adapter(context, SITE_ROOT_INFOS_KEY,
                                  ISiteRootInfos, name='++infos++')


@adapter_config(name='infos',
                required=ISiteRoot, provides=ITraversable)
class SiteRootInfosTraverser(ContextAdapter):
    """Site root infos traverser"""

    def traverse(self, name, furtherPath=None):
        """Namespace traverser"""
        return ISiteRootInfos(self.context)


@implementer(ISiteRootRoles)
class SiteRootRoles(ProtectedObjectRoles):
    """Site root roles"""

    webmasters = RolePrincipalsFieldProperty(ISiteRootRoles['webmasters'])
    designers = RolePrincipalsFieldProperty(ISiteRootRoles['designers'])
    operators = RolePrincipalsFieldProperty(ISiteRootRoles['operators'])

    def get_operators_group(self):
        """Get operators group"""
        if not self.operators:
            return None
        sm = get_utility(ISecurityManager)
        return sm.get_raw_principal(next_from(self.operators))


@adapter_config(required=ISiteRoot,
                provides=ISiteRootRoles)
def site_root_roles_adapter(context):
    """Site root roles adapters"""
    return SiteRootRoles(context)


@adapter_config(name=SITEROOT_ROLES,
                required=ISiteRoot,
                provides=IRolesPolicy)
class SiteRootRolesPolicy(ContextAdapter):
    """Site root roles policy"""

    roles_interface = ISiteRootRoles
    weight = 10


@adapter_config(required=(ISiteRoot, IPyAMSLayer, Interface),
                provides=IViewContextPermissionChecker)
@adapter_config(required=(ISiteRootInfos, IPyAMSLayer, Interface),
                provides=IViewContextPermissionChecker)
class SiteRootPermissionChecker(ContextRequestViewAdapter):
    """Site root permission checker"""

    edit_permission = MANAGE_SITE_ROOT_PERMISSION


@subscriber(IGrantedRoleEvent)
def handle_granted_role(event):
    """Handle granted role event"""
    role = get_utility(IRole, name=event.role_id)
    if (role is None) or not role.custom_data.get('set_as_operator', True):
        return
    root = get_parent(event.object, ISiteRoot)
    if root is None:
        return
    roles = ISiteRootRoles(root)
    group = roles.get_operators_group()
    if (group is None) or not ILocalGroup.providedBy(group):
        return
    group.principals |= {event.principal_id}



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.merge module

This module defines several views *mergers*, which can utilities which can be
used to merge several views items together.
"""

from heapq import merge
from itertools import chain, zip_longest
from random import shuffle

from zope.componentvocabulary.vocabulary import UtilityTerm, UtilityVocabulary
from zope.dublincore.interfaces import IZopeDublinCore

from pyams_content.shared.view.interfaces import CREATION_DATE_ORDER, FIRST_PUBLICATION_DATE_ORDER, \
    PUBLICATION_DATE_ORDER, UPDATE_DATE_ORDER
from pyams_content.shared.view.interfaces.query import MergeModes, IViewsMerger, VIEWS_MERGERS_VOCABULARY
from pyams_utils.registry import utility_config
from pyams_utils.request import check_request
from pyams_utils.vocabulary import vocabulary_config
from pyams_workflow.interfaces import IWorkflowPublicationInfo

__docformat__ = 'restructuredtext'

from pyams_content import _


@vocabulary_config(name=VIEWS_MERGERS_VOCABULARY)
class ViewsMergersVocabulary(UtilityVocabulary):
    """Views mergers vocabulary"""

    interface = IViewsMerger
    nameOnly = True

    def __init__(self, context, **kw):
        request = check_request()
        registry = request.registry
        translate = request.localizer.translate
        utils = [
            (name, translate(util.label))
            for (name, util) in registry.getUtilitiesFor(self.interface)
        ]
        self._terms = dict((title, UtilityTerm(name, title)) for name, title in utils)


class SingleViewMergeMode:
    """Single view merger"""

    @classmethod
    def get_results(cls, view, context, ignore_cache=False, request=None,
                    aggregates=None, settings=None, get_count=False):
        count, aggregations, results = view.get_results(context, ignore_cache=ignore_cache, request=request,
                                                        aggregates=aggregates, settings=settings, get_count=True)
        if get_count:
            yield count
            yield aggregations
        yield from results


@utility_config(name=MergeModes.CONCAT.value,
                provides=IViewsMerger)
class ViewsConcatenateMergeMode(SingleViewMergeMode):
    """Views concatenate merge mode"""

    label = _("Concatenate views items in order")

    @classmethod
    def get_results(cls, views, context, ignore_cache=False, request=None,
                    aggregates=None, settings=None, get_count=False):
        views = list(views) or ()
        if len(views) == 1:
            yield from super().get_results(views[0], context, ignore_cache=ignore_cache, request=request,
                                           aggregates=aggregates, settings=settings, get_count=get_count)
        else:
            results = (
                view.get_results(context, ignore_cache=ignore_cache, request=request,
                                 aggregates=aggregates, settings=settings, get_count=False)
                for view in views
            )
            yield from chain(*results)


@utility_config(name=MergeModes.RANDOM.value,
                provides=IViewsMerger)
class ViewsRandomMergeMode(SingleViewMergeMode):
    """Views random merge mode"""

    label = _("Extract items randomly")

    @classmethod
    def get_results(cls, views, context, ignore_cache=False, request=None,
                    aggregates=None, settings=None, get_count=False):
        views = list(views) or ()
        if len(views) == 1:
            yield from super().get_results(views[0], context, ignore_cache=ignore_cache, request=request,
                                           aggregates=aggregates, settings=settings, get_count=get_count)
        else:
            results = (
                view.get_results(context, ignore_cache=ignore_cache, request=request,
                                 aggregates=aggregates, settings=settings)
                for view in views
            )
            results = list(chain(*results))
            shuffle(results)
            yield from iter(results)


@utility_config(name=MergeModes.ZIP.value,
                provides=IViewsMerger)
class ViewsZipMergeMode(SingleViewMergeMode):
    """Views zip merge mode"""

    label = _("Take items from views one by one, in views order")

    @classmethod
    def get_results(cls, views, context, ignore_cache=False, request=None,
                    aggregates=None, settings=None, get_count=False):
        views = list(views) or ()
        if len(views) == 1:
            yield from super().get_results(views[0], context, ignore_cache=ignore_cache, request=request,
                                           aggregates=aggregates, settings=settings, get_count=get_count)
        else:
            results = (
                view.get_results(context, ignore_cache=ignore_cache, request=request,
                                 aggregates=aggregates, settings=settings)
                for view in views
            )
            for array in zip_longest(*results):
                yield from filter(lambda x: x is not None, array)


@utility_config(name=MergeModes.RANDOM_ZIP.value,
                provides=IViewsMerger)
class ViewsRandomZipMergeMode(SingleViewMergeMode):
    """Views random zip merge mode"""

    label = _("Take items from views one by one, in random order")

    @classmethod
    def get_results(cls, views, context, ignore_cache=False, request=None,
                    aggregates=None, settings=None, get_count=False):
        views = list(views) or ()
        if len(views) == 1:
            yield from super().get_results(views[0], context, ignore_cache=ignore_cache, request=request,
                                           aggregates=aggregates, settings=settings, get_count=get_count)
        else:
            results = [
                view.get_results(context, ignore_cache=ignore_cache, request=request,
                                 aggregates=aggregates, settings=settings)
                for view in views
            ]
            shuffle(results)
            for array in zip_longest(*results):
                yield from filter(lambda x: x is not None, array)


class SortedMergeMode(SingleViewMergeMode):
    """Sorted merge mode base class"""

    label = None
    sort_index = None
    sort_key = None

    @classmethod
    def get_results(cls, views, context, ignore_cache=False, request=None,
                    aggregates=None, settings=None, get_count=False):
        views = list(views) or ()
        if len(views) == 1:
            yield from super().get_results(views[0], context, ignore_cache=ignore_cache, request=request,
                                           aggregates=aggregates, settings=settings, get_count=get_count)
        else:
            results = (
                sorted(view.get_results(context, ignore_cache=ignore_cache, request=request,
                                        aggregates=aggregates, settings=settings),
                       key=cls.sort_key, reverse=True)
                for view in views
            )
            yield from merge(*results, key=cls.sort_key, reverse=True)


@utility_config(name=f'{CREATION_DATE_ORDER}.sort',
                provides=IViewsMerger)
class CreationDateSortedMergeMode(SortedMergeMode):
    """Merge pre-sorted views by creation date"""

    label = _("Sort all results by creation date")
    sort_index = CREATION_DATE_ORDER

    @staticmethod
    def sort_key(item):
        return IZopeDublinCore(item).created


@utility_config(name=f'{UPDATE_DATE_ORDER}.sort',
                provides=IViewsMerger)
class UpdateDateSortedMergeMode(SortedMergeMode):
    """Merge pre-sorted views by last update date"""

    label = _("Sort all results by last update date")
    sort_index = UPDATE_DATE_ORDER

    @staticmethod
    def sort_key(item):
        return IZopeDublinCore(item).modified


@utility_config(name=f'{PUBLICATION_DATE_ORDER}.sort',
                provides=IViewsMerger)
class PublicationDateSortedMergeMode(SortedMergeMode):
    """Merge pre-sorted views by publication date"""

    label = _("Sort all results by current publication date")
    sort_index = PUBLICATION_DATE_ORDER

    @staticmethod
    def sort_key(item):
        return IWorkflowPublicationInfo(item).publication_date


@utility_config(name=f'{FIRST_PUBLICATION_DATE_ORDER}.sort',
                provides=IViewsMerger)
class FirstPublicationDateSortedMergeMode(SortedMergeMode):
    """Merge pre-sorted views by first publication date"""

    label = _("Sort all results by first publication date")
    sort_index = FIRST_PUBLICATION_DATE_ORDER

    @staticmethod
    def sort_key(item):
        return IWorkflowPublicationInfo(item).first_publication_date



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration.interfaces module

This module defines interfaces related to illustrations management.
"""

__docformat__ = 'restructuredtext'

from zope.annotation import IAttributeAnnotatable
from zope.interface import Interface
from zope.schema import Choice, TextLine

from pyams_content.component.paragraph import IBaseParagraph
from pyams_content.component.paragraph.interfaces import ParagraphRendererChoice
from pyams_content.feature.renderer import IRenderedContent
from pyams_file.schema import I18nThumbnailMediaField
from pyams_i18n.schema import I18nTextField, I18nTextLineField

from pyams_content import _


BASIC_ILLUSTRATION_KEY = 'pyams_content.illustration.base'

ILLUSTRATION_KEY = 'pyams_content.illustration'
ILLUSTRATION_RENDERERS = 'pyams.illustration.renderers'

LINK_ILLUSTRATION_KEY = '{0}::link'.format(ILLUSTRATION_KEY)


class IBaseIllustration(Interface):
    """Basic illustration interface"""

    data = I18nThumbnailMediaField(title=_("Image or video data"),
                                   description=_("Image or video content"),
                                   required=False)

    def has_data(self):
        """Check if data is provided in any language"""

    title = I18nTextLineField(title=_("Legend"),
                              required=False)

    alt_title = I18nTextLineField(title=_("Accessibility title"),
                                  description=_("Alternate title used to describe image content"),
                                  required=False)

    author = TextLine(title=_("Author"),
                      description=_("Name of picture's author"),
                      required=False)


class IIllustration(IBaseIllustration, IRenderedContent):
    """Illustration paragraph"""

    description = I18nTextField(title=_("Associated text"),
                                description=_("Illustration description displayed in "
                                              "front-office templates"),
                                required=False)

    renderer = Choice(title=_("Illustration renderer"),
                      description=_("Renderer used to display illustration; please note that "
                                    "this renderer is not always used, but only in some specific "
                                    "contexts"),
                      vocabulary=ILLUSTRATION_RENDERERS,
                      default='default')


class IParagraphIllustration(IIllustration):
    """Paragraph illustration marker interface"""


class ILinkIllustration(IBaseIllustration):
    """Navigation link illustration interface"""


class IIllustrationTargetBase(IAttributeAnnotatable):
    """Illustration base target interface"""


class IBaseIllustrationTarget(IIllustrationTargetBase):
    """Basic illustration target marker interface

    This interface is used to identify contexts which should handle only
    *basic* illustrations.
    """


class IIllustrationTarget(IBaseIllustrationTarget):
    """Base illustration target marker interface"""


class ILinkIllustrationTarget(IIllustrationTargetBase):
    """Link illustration target interface"""


#
# Illustration paragraph
#

ILLUSTRATION_PARAGRAPH_TYPE = 'illustration'
ILLUSTRATION_PARAGRAPH_NAME = _("Illustration")
ILLUSTRATION_PARAGRAPH_RENDERERS = 'PyAMS_content.paragraph.illustration.renderers'
ILLUSTRATION_PARAGRAPH_ICON_CLASS = 'fas fa-image'


class IIllustrationParagraph(IBaseIllustration, IBaseParagraph):
    """Illustration paragraph interface"""

    data = I18nThumbnailMediaField(title=_("Image or video data"),
                                   description=_("Image or video content"),
                                   required=True)

    description = I18nTextField(title=_("Associated text"),
                                description=_("Illustration description displayed in "
                                              "front-office templates"),
                                required=False)

    renderer = ParagraphRendererChoice(description=_("Presentation template used for "
                                                     "illustration"),
                                       renderers=ILLUSTRATION_PARAGRAPH_RENDERERS)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.reference.pictogram.zmi.widget module

"""

from zope.interface import implementer

from pyams_content.reference.pictogram import IPictogramTable
from pyams_content.reference.pictogram.zmi import get_pictogram_header
from pyams_form.browser.select import SelectWidget
from pyams_form.widget import FieldWidget
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.registry import query_utility
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import RawContentProvider


__docformat__ = 'restructuredtext'

from pyams_content import _


@implementer(IObjectData)
class PictogramSelectWidget(SelectWidget):
    """Pictogram selection widget"""

    no_value_message = _("No selected pictogram")

    pictograms = None
    label_id = None
    suffix = None

    def update(self):
        super().update()
        self.label_id = '{0}_header'.format(self.id)
        self.pictograms = query_utility(IPictogramTable)
        if self.value and (self.pictograms is not None):
            pictogram = self.pictograms.get(self.value[0])
            if pictogram is not None:
                self.suffix = RawContentProvider(
                    self.context, self.request,
                    html=f'<span id="{self.label_id}"'
                         f' class="text-info">'
                         f'{get_pictogram_header(pictogram, self.request)}'
                         f'</span>')
                return
        header = self.request.localizer.translate(_("Default header: --"))
        self.suffix = RawContentProvider(
            self.context, self.request,
            html=f'<span id="{self.label_id}"'
                 f' class="text-info">{header}</span>')

    @property
    def object_data(self):
        """Object data getter"""
        return {
            'ams-change-handler': 'MyAMS.helpers.select2ChangeHelper',
            'ams-stop-propagation': 'true',
            'ams-select2-helper-type': 'html',
            'ams-select2-helper-url': absolute_url(self.pictograms, self.request,
                                                   'get-pictogram-header.html'),
            'ams-select2-helper-target': '#{0}'.format(self.label_id)
        }


def PictogramSelectFieldWidget(field, request):  # pylint: disable=invalid-name
    """Pictogram selection field widget factory"""
    return FieldWidget(field, PictogramSelectWidget(request))



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.workflow.zmi.header module

This module defines workflow-related content headers.
"""

from datetime import datetime, timezone

from pyams_content.shared.site.interfaces import IBaseSiteItem
from pyams_sequence.interfaces import ISequentialIdTarget
from pyams_skin.interfaces.view import IModalPage
from pyams_skin.interfaces.viewlet import IHeaderViewletManager
from pyams_template.template import override_template
from pyams_utils.adapter import NullAdapter
from pyams_utils.date import format_datetime
from pyams_utils.timezone import tztime
from pyams_viewlet.viewlet import Viewlet, viewlet_config
from pyams_workflow.interfaces import IWorkflowPublicationInfo
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.zmi.viewlet.header import ContentHeaderViewlet

__docformat__ = 'restructuredtext'

from pyams_content import _


override_template(ContentHeaderViewlet,
                  template='templates/sequence-header.pt',
                  context=ISequentialIdTarget, layer=IAdminLayer)


@viewlet_config(name='workflow-status',
                context=IBaseSiteItem, layer=IAdminLayer,
                manager=IHeaderViewletManager, weight=20)
class WorkflowPublicationSupportHeaderViewlet(Viewlet):
    """Workflow publication support header viewlet"""

    def __new__(cls, context, request, view, manager):
        if IModalPage.providedBy(view):
            return None
        return Viewlet.__new__(cls)

    def render(self):
        """Status getter"""
        translate = self.request.localizer.translate
        now = tztime(datetime.now(timezone.utc))
        pub_info = IWorkflowPublicationInfo(self.context)
        if pub_info.publication_effective_date:
            if pub_info.publication_effective_date <= now:
                if pub_info.publication_expiration_date:
                    if pub_info.publication_effective_date > now:
                        state = _("Retired")
                        state_label = _("{state} since {from_date}")
                    else:
                        state = _("Published")
                        state_label = _("{state} since {from_date} until {to_date}")
                else:
                    state = _("Published")
                    state_label = _("{state} since {from_date}")
            else:
                if pub_info.publication_expiration_date:
                    state = _("To be published")
                    state_label = _("{state} from {from_date} to {to_date}")
                else:
                    state = _("Published")
                    state_label = _("{state} after {from_date}")
            state = translate(state_label).format(
                state='<span class="text-danger">{}</span>'.format(translate(state)),
                from_date=format_datetime(pub_info.publication_effective_date),
                to_date=format_datetime(pub_info.publication_expiration_date))
        else:
            state = '<span class="text-danger">{}</span>'.format(
                translate(_("Not published")))
        return '<div class="mb-1">{}</div>'.format(state)


@viewlet_config(name='workflow-status',
                context=IBaseSiteItem, layer=IAdminLayer, view=IModalPage,
                manager=IHeaderViewletManager, weight=20)
class WorkflowPublicationSupportModalHeaderViewlet(NullAdapter):
    """Workflow publication support modal header viewlet"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.zmi.viewlet.toplinks module

This module provides top-menus viewlets.
"""

from zope.interface import Interface

from pyams_viewlet.manager import viewletmanager_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import ITopLinksViewletManager
from pyams_zmi.zmi.viewlet.toplinks import TopMenusGroupViewletManager


__docformat__ = 'restructuredtext'

from pyams_content import _


@viewletmanager_config(name='top-tabs',
                       context=Interface, layer=IAdminLayer,
                       manager=ITopLinksViewletManager, weight=10)
class TopTabsViewletManager(TopMenusGroupViewletManager):
    """Top tabs viewlet manager"""

    label = _("Shortcuts")



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""
from pyams_content.component.association.interfaces import IAssociationParagraph
from pyams_content.component.association.skin import AssociationContainerRendererMixin
from pyams_content.feature.renderer import DefaultContentRenderer, \
    IContentRenderer
from pyams_portal.interfaces import DEFAULT_RENDERER_NAME
from pyams_layer.interfaces import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config


__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(name=DEFAULT_RENDERER_NAME,
                required=(IAssociationParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/association-default.pt', layer=IPyAMSLayer)
class AssociationParagraphDefaultRenderer(AssociationContainerRendererMixin,
                                          DefaultContentRenderer):
    """Association paragraph default renderer"""

    label = _("Associations list (default)")



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph module

Paragraphs are components which are used to build the *content* of a given web page;
these components can be very simple elements like rich HTML text, but can also include
specialized components like verbatims, contact cards, framed text, illustrations, images
galleries, videos or any other kind of content that you can imagine...
"""

__docformat__ = 'restructuredtext'

from persistent import Persistent
from pyramid.events import subscriber
from zope.container.contained import Contained
from zope.interface import implementer
from zope.lifecycleevent import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent, \
    ObjectModifiedEvent
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.component.paragraph.interfaces import CONTENT_PARAGRAPHS_VOCABULARY, \
    IBaseParagraph, IParagraphContainer, IParagraphContainerTarget, IParagraphTitle, \
    PARAGRAPH_FACTORIES_VOCABULARY
from pyams_content.feature.preview.interfaces import IPreviewTarget
from pyams_content.feature.renderer import RenderedContentMixin
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION, PUBLISH_CONTENT_PERMISSION
from pyams_i18n.interfaces import II18n
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.factory import get_all_factories
from pyams_utils.html import html_to_text
from pyams_utils.registry import get_pyramid_registry
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config
from pyams_zmi.interfaces import IObjectHint, IObjectIcon, IObjectLabel


@vocabulary_config(name=PARAGRAPH_FACTORIES_VOCABULARY)
class ParagraphFactoriesVocabulary(SimpleVocabulary):
    """Paragraph factories vocabulary"""

    def __init__(self, context=None):  # pylint: disable=unused-argument
        request = check_request()
        translate = request.localizer.translate
        terms = sorted([
            SimpleTerm(name, title=translate(factory.factory_label))
            for name, factory in get_all_factories(IBaseParagraph)
        ], key=lambda x: x.title)
        super().__init__(terms)


@vocabulary_config(name=CONTENT_PARAGRAPHS_VOCABULARY)
class ContentParagraphsVocabulary(SimpleVocabulary):
    """Content paragraphs vocabulary"""

    def __init__(self, context):

        def get_title(paragraph):
            """Paragraph title getter"""
            adapter = request.registry.queryMultiAdapter((paragraph, request),
                                                         IParagraphTitle)
            if adapter is not None:
                return html_to_text(adapter)
            return II18n(paragraph).query_attribute('title', request=request) or \
                BaseParagraph.empty_title

        request = check_request()
        if not IParagraphContainerTarget.providedBy(context):
            context = get_parent(context, IParagraphContainerTarget)
        if context is not None:
            terms = [
                SimpleTerm(para.__name__,
                           title=f'{index+1}: {get_title(para)}')
                for index, para in enumerate(IParagraphContainer(context).values())
            ]
        else:
            terms = []
        super().__init__(terms)


@implementer(IBaseParagraph, IPreviewTarget)
class BaseParagraph(RenderedContentMixin, Persistent, Contained):
    """Base paragraph persistent class"""

    factory_name = None
    factory_label = None
    factory_intf = None
    secondary = False

    icon_class = None

    @property
    def icon_hint(self):
        """Icon hint getter"""
        request = check_request()
        return request.localizer.translate(self.factory_label)

    visible = FieldProperty(IBaseParagraph['visible'])
    anchor = FieldProperty(IBaseParagraph['anchor'])
    locked = FieldProperty(IBaseParagraph['locked'])
    title = FieldProperty(IBaseParagraph['title'])

    empty_title = ' -' * 8


@adapter_config(required=IBaseParagraph,
                provides=IObjectHint)
def paragraph_hint(context):
    """Paragraph hint getter"""
    return context.icon_hint


@adapter_config(required=IBaseParagraph,
                provides=IObjectIcon)
def paragraph_icon(context):
    """Paragraph icon getter"""
    return context.icon_class


@adapter_config(required=IBaseParagraph,
                provides=IObjectLabel)
def paragraph_label(context):
    """Paragraph label getter"""
    request = check_request()
    title = II18n(context).query_attribute('title', request=request)
    return title or BaseParagraph.empty_title


@adapter_config(required=IBaseParagraph,
                provides=IViewContextPermissionChecker)
@adapter_config(required=IParagraphContainer,
                provides=IViewContextPermissionChecker)
class ParagraphPermissionChecker(ContextAdapter):
    """Paragraph permission checker"""

    @property
    def edit_permission(self):
        """Edit permission checker"""
        parent = get_parent(self.context, IParagraphContainerTarget)
        if parent is not None:
            return IViewContextPermissionChecker(parent).edit_permission
        return None


@adapter_config(name='delete',
                required=IBaseParagraph,
                provides=IViewContextPermissionChecker)
class ParagraphDeletePermissionChecker(ParagraphPermissionChecker):
    """Paragraph delete permission checker"""

    @property
    def edit_permission(self):
        """Edit permission checker"""
        permission = super().edit_permission
        if self.context.locked and (permission == MANAGE_CONTENT_PERMISSION):
            return PUBLISH_CONTENT_PERMISSION
        return permission


@subscriber(IObjectAddedEvent, context_selector=IBaseParagraph)
@subscriber(IObjectModifiedEvent, context_selector=IBaseParagraph)
@subscriber(IObjectRemovedEvent, context_selector=IBaseParagraph)
def handle_paragraph_event(event):
    """Handle added paragraph"""
    content = get_parent(event.object, IParagraphContainerTarget)
    if content is not None:
        get_pyramid_registry().notify(ObjectModifiedEvent(content))



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.sitemap module

This module defines main sitemap management components.
"""

from datetime import datetime, timezone
from itertools import product

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import Any, Eq
from pyramid.traversal import resource_path
from pyramid.view import view_config
from zope.intid import IIntIds
from zope.schema.vocabulary import getVocabularyRegistry

from pyams_catalog.query import CatalogResultSet
from pyams_content.feature.seo import ISEOContentInfo
from pyams_content.feature.sitemap.interfaces import ISitemapExtension
from pyams_content.root import ISiteRootToolsConfiguration
from pyams_content.shared.common import IBaseSharedTool, SHARED_CONTENT_TYPES_VOCABULARY
from pyams_i18n.interfaces import II18nManager
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_site.interfaces import ISiteRoot
from pyams_utils.list import unique_iter
from pyams_utils.registry import get_utilities_for, get_utility
from pyams_utils.timezone import tztime
from pyams_workflow.interfaces import IWorkflow, IWorkflowPublicationInfo

__docformat__ = 'restructuredtext'


@view_config(name='robots.txt',
             context=ISiteRoot, request_type=IPyAMSUserLayer,
             renderer='templates/robots.pt')
def site_root_robots_view(request):
    """Site root robots.txt view"""
    request.response.content_type = 'text/plain'
    disallow = []
    for name, tool in get_utilities_for(IBaseSharedTool):
        if not name:
            continue
        seo_info = ISEOContentInfo(tool, None)
        if seo_info is None:
            if not tool.shared_content_menu:
                disallow.append(resource_path(tool))
                continue
        else:
            if not seo_info.include_sitemap:
                disallow.append(resource_path(tool))
                continue
        publication_info = IWorkflowPublicationInfo(tool, None)
        if (publication_info is not None) and not publication_info.is_visible(request):
            disallow.append(resource_path(tool))
    return {
        'tools_configuration': ISiteRootToolsConfiguration(request.root),
        'disallow': disallow
    }


@view_config(name='humans.txt',
             context=ISiteRoot, request_type=IPyAMSUserLayer,
             renderer='templates/humans.pt')
def site_root_humans_view(request):
    """Site root humans.txt view"""
    request.response.content_type = 'text/plain'
    return {}


@view_config(name='sitemap.xml',
             context=ISiteRoot, request_type=IPyAMSUserLayer,
             renderer='templates/root-sitemap.pt')
class SiteRootSitemapView:
    """Site root sitemap view"""

    def __init__(self, request):
        self.request = request

    def __call__(self):
        self.request.response.content_type = 'text/xml'
        return {}

    @property
    def sources(self):
        """Sitemap sources"""
        request = self.request
        timestamp = tztime(datetime.now(timezone.utc)).isoformat()
        for name, tool in get_utilities_for(IBaseSharedTool):
            if not name:
                continue
            seo_info = ISEOContentInfo(tool, None)
            if seo_info is None:
                if not tool.shared_content_menu:
                    continue
            else:
                if not seo_info.include_sitemap:
                    continue
            publication_info = IWorkflowPublicationInfo(tool, None)
            if (publication_info is None) or publication_info.is_visible(request):
                yield timestamp, tool
        for name, adapter in request.registry.getAdapters((request.context, request),
                                                          ISitemapExtension):
            source = adapter.source
            if source is not None:
                yield timestamp, source


@view_config(name='sitemap.xml',
             context=IBaseSharedTool, request_type=IPyAMSUserLayer,
             renderer='templates/tool-sitemap.pt')
class SharedToolSitemapView:
    """Shared tool sitemap view"""

    def __init__(self, request):
        self.request = request

    def __call__(self):
        self.request.response.content_type = 'text/xml'
        return {}

    @property
    def contents(self):
        """Sitemap contents getter"""
        context = self.request.context
        catalog = get_utility(ICatalog)
        intids = get_utility(IIntIds)
        workflow = IWorkflow(context)
        vocabulary = getVocabularyRegistry().get(context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = Eq(catalog['parents'], intids.register(context)) & \
            Any(catalog['content_type'], vocabulary.by_value.keys()) & \
            Any(catalog['workflow_state'], workflow.visible_states)
        for version in unique_iter(CatalogResultSet(CatalogQuery(catalog).query(params))):
            seo_info = ISEOContentInfo(version, None)
            if (seo_info is None) or seo_info.include_sitemap:
                yield from product(II18nManager(version).get_languages(), (version,))



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.zmi.interfaces module

Common management interfaces.
"""

from pyramid.interfaces import IView
from zope.interface import Interface


__docformat__ = 'restructuredtext'


class IDashboardView(IView):
    """Dashboard view marker interface"""


class IDashboardTable(Interface):
    """Dashboard table marker interface"""


class IDashboardColumn(Interface):
    """Dashboard column marker interface"""


class IDashboardContentVisibility(Interface):
    """Dashboard content visibility column provider interface

    Adapters to this interface should return a tuple containing a
    boolean value to specify if the generated action icon should be
    active to switch item visibility, and a string containing the
    icon HTML definition.
    """


class IDashboardContentLabel(Interface):
    """Dashboard content label column provider interface"""


class IDashboardContentType(Interface):
    """Dashboard content type column provider interface"""


class ISiteRootDashboardContentType(IDashboardContentType):
    """Site root dashboard content type column provider interface"""


class IDashboardContentNumber(Interface):
    """Dashboard content reference number column provider interface"""


class IDashboardContentStatus(Interface):
    """Dashboard content status column provider interface"""


class IDashboardContentStatusDatetime(Interface):
    """Dashboard content status datetime provider interface"""


class IDashboardContentVersion(Interface):
    """Dashboard content version column provider interface"""


class IDashboardContentModifier(Interface):
    """Dashboard content modifier column provider interface"""


class IDashboardContentOwner(Interface):
    """Dashboard content owner column provider interface"""


class IDashboardContentTimestamp(Interface):
    """Dashboard content modification timestamp column provider interface"""


class IMyDashboardMenu(Interface):
    """My dashboard menu marker interface"""


class IAllDashboardMenu(Interface):
    """General dashboard menu"""



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.schema import Bool

from pyams_portal.interfaces import IPortletSettings

__docformat__ = 'restructuredtext'

from pyams_content import _


PAGE_HEADER_PORTLET_NAME = 'pyams_content.portlet.page_header'
PAGE_HEADER_ICON_CLASS = 'fas fa-user-circle'


class IPageHeaderPortletSettings(IPortletSettings):
    """Page header portlet settings interface"""

    display_logo = Bool(title=_("Display logo"),
                        description=_("Uncheck option to disable logo display"),
                        required=True,
                        default=True)

    display_context_title = Bool(title=_("Display context title"),
                                 description=_("Title is extracted from context"),
                                 required=True,
                                 default=True)

    display_profile_link = Bool(title=_("Display profile link"),
                                description=_("Display link to get access to login form "
                                              "or user profile"),
                                required=True,
                                default=True)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery.zmi.paragraph module

This module defines management interface components which are used to handle
galleries paragraphs.
"""

from pyams_content.component.gallery import IBaseGallery, IGallery, IGalleryTarget
from pyams_content.component.gallery.interfaces import GALLERY_PARAGRAPH_ICON_CLASS, \
    GALLERY_PARAGRAPH_NAME, GALLERY_PARAGRAPH_TYPE, IGalleryParagraph
from pyams_content.component.paragraph import IParagraphContainer, IParagraphContainerTarget
from pyams_content.component.paragraph.zmi import BaseParagraphAddForm, BaseParagraphAddMenu, \
    IParagraphContainerBaseTable, IParagraphContainerFullTable
from pyams_content.component.paragraph.zmi.container import ParagraphTitleToolbarItemMixin
from pyams_content.component.paragraph.zmi.interfaces import IParagraphTitleToolbar
from pyams_zmi.interfaces.form import IPropertiesEditForm
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IFormFields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.zmi.widget import RendererSelectFieldWidget
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager


__docformat__ = 'restructuredtext'


@viewlet_config(name='gallery',
                context=IGalleryTarget, layer=IAdminLayer,
                view=IParagraphContainerFullTable, manager=IParagraphTitleToolbar,
                weight=15)
@viewlet_config(name='gallery',
                context=IGalleryParagraph, layer=IAdminLayer,
                view=IParagraphContainerFullTable, manager=IParagraphTitleToolbar,
                weight=15)
class GalleryParagraphTitleToolbarViewlet(ParagraphTitleToolbarItemMixin):
    """Gallery paragraph marker toolbar viewlet"""

    icon_class = GALLERY_PARAGRAPH_ICON_CLASS
    icon_hint = GALLERY_PARAGRAPH_NAME

    target_intf = IBaseGallery

    def update(self):
        gallery = self.target_intf(self.context, None)
        if gallery is not None:
            self.counter = len(gallery.values())


@viewlet_config(name='add-gallery-paragraph.menu',
                context=IParagraphContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerBaseTable,
                manager=IContextAddingsViewletManager, weight=65)
class GalleryParagraphAddMenu(BaseParagraphAddMenu):
    """Gallery paragraph add menu"""

    label = GALLERY_PARAGRAPH_NAME
    icon_class = GALLERY_PARAGRAPH_ICON_CLASS

    factory_name = GALLERY_PARAGRAPH_TYPE
    href = 'add-gallery-paragraph.html'


@ajax_form_config(name='add-gallery-paragraph.html',
                  context=IParagraphContainer, layer=IPyAMSLayer)
class GalleryParagraphAddForm(BaseParagraphAddForm):
    """Gallery paragraph add form"""

    content_factory = IGalleryParagraph


@adapter_config(required=(IParagraphContainer, IAdminLayer, GalleryParagraphAddForm),
                provides=IFormFields)
@adapter_config(required=(IGalleryParagraph, IAdminLayer, IPropertiesEditForm),
                provides=IFormFields)
def gallery_form_fields(context, request, form):
    """Gallery form fields"""
    fields = Fields(IGalleryParagraph).select('title', 'renderer')
    if IGalleryParagraph.providedBy(context):
        fields['renderer'].widget_factory = RendererSelectFieldWidget
    return fields



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.restrictions module

This modules defines components which are used to handle contributors and managers
*restrictions*. These restrictions are rules which can be defined and extended using
adapters to define access rules.
"""

from persistent import Persistent
from pyramid.events import subscriber
from zope.container.contained import Contained
from zope.container.folder import Folder
from zope.interface import alsoProvides
from zope.schema.fieldproperty import FieldProperty

from pyams_content.interfaces import CONTRIBUTOR_ROLE, MANAGER_ROLE
from pyams_content.shared.common import IBaseSharedTool, ISharedContent, IWfSharedContent
from pyams_content.shared.common.interfaces import CONTRIBUTORS_RESTRICTIONS_KEY, \
    CONTRIBUTOR_WORKFLOW_RESTRICTIONS_KEY, IContributorRestrictions, \
    IContributorWorkflowRestrictions, IManagerRestrictions, IManagerWorkflowRestrictions, \
    IPrincipalRestrictions, IRestrictionInfo, ISharedToolRestrictions, IWfSharedContentRoles, \
    MANAGERS_RESTRICTIONS_KEY, MANAGER_WORKFLOW_RESTRICTIONS_KEY
from pyams_security.interfaces import IGrantedRoleEvent, IRevokedRoleEvent
from pyams_security.interfaces.base import IPrincipalInfo
from pyams_utils.adapter import adapter_config, get_adapter_weight, get_annotation_adapter
from pyams_utils.factory import factory_config, get_object_factory
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent


__docformat__ = 'restructuredtext'


@factory_config(IPrincipalRestrictions)
class PrincipalRestrictionInfo(Persistent, Contained):
    """Principal restriction info"""

    principal_id = FieldProperty(IPrincipalRestrictions['principal_id'])

    def __init__(self, principal_id):
        self.principal_id = principal_id


@factory_config(provided=ISharedToolRestrictions)
class SharedToolRestrictions(Folder):
    """Shared tool restrictions persistent class"""

    def new_restrictions(self, principal, interface=None):
        """New principal restrictions factory"""
        if IPrincipalInfo.providedBy(principal):
            principal = principal.id
        factory = get_object_factory(IPrincipalRestrictions)
        restrictions = factory(principal)
        if interface is not None:
            alsoProvides(restrictions, interface)
        return restrictions

    def set_restrictions(self, principal, restrictions=None, interface=None):
        """Set new restrictions for given principal"""
        if IPrincipalInfo.providedBy(principal):
            principal = principal.id
        if principal not in self:
            if restrictions is None:
                restrictions = self.new_restrictions(principal, interface)
            self[principal] = restrictions

    def get_restrictions(self, principal, create_if_none=False):
        """Principal restrictions getter"""
        if IPrincipalInfo.providedBy(principal):
            principal = principal.id
        restrictions = self.get(principal)
        if (restrictions is None) and create_if_none:
            restrictions = self.new_restrictions(principal)
            self.set_restrictions(principal, restrictions)
        return restrictions

    def drop_restrictions(self, principal):
        """Drop restrictions for given principal"""
        if IPrincipalInfo.providedBy(principal):
            principal = principal.id
        if principal in self:
            del self[principal]

    def can_access(self, context, permission, request=None):
        """Principal access checker"""
        if request is None:
            request = check_request()
        if permission and not request.has_permission(permission, context=context):
            return False
        restrictions = self.get_restrictions(request.principal, create_if_none=False)
        if restrictions:
            for _name, adapter in sorted(request.registry.getAdapters((restrictions,),
                                                                      IRestrictionInfo),
                                         key=get_adapter_weight):
                if adapter.can_access(context, permission, request):
                    return True
        return False


#
# Contributor restrictions
#

@adapter_config(required=IBaseSharedTool,
                provides=IContributorRestrictions)
def shared_tool_contributor_restrictions(context):
    """Shared tool contributor restrictions"""
    return get_annotation_adapter(context, CONTRIBUTORS_RESTRICTIONS_KEY,
                                  ISharedToolRestrictions)


@factory_config(IContributorWorkflowRestrictions)
class ContributorWorkflowRestrictions(Persistent):
    """Contributor workflow restrictions"""

    weight = 10

    show_workflow_warning = FieldProperty(
        IContributorWorkflowRestrictions['show_workflow_warning'])
    owners = FieldProperty(IContributorWorkflowRestrictions['owners'])

    def can_access(self, context, permission, request):  # pylint: disable=unused-argument
        """Access checker"""
        roles = IWfSharedContentRoles(context, None)
        if roles is None:
            return False
        return bool(roles.owner & set(self.owners or ()))


@adapter_config(name='workflow',
                required=IContributorRestrictions,
                provides=IRestrictionInfo)
@adapter_config(required=IPrincipalRestrictions,
                provides=IContributorWorkflowRestrictions)
def contributor_workflow_restrictions_adapter(context):
    """Base contributor restrictions adapter"""
    return get_annotation_adapter(context, CONTRIBUTOR_WORKFLOW_RESTRICTIONS_KEY,
                                  IContributorWorkflowRestrictions)


@adapter_config(required=ISharedContent,
                provides=IContributorRestrictions)
@adapter_config(required=IWfSharedContent,
                provides=IContributorRestrictions)
def shared_content_contributor_restrictions(context):
    """Shared tool contributor restrictions"""
    tool = get_parent(context, IBaseSharedTool)
    if tool is not None:
        return IContributorRestrictions(tool)
    return None


@subscriber(IGrantedRoleEvent, role_selector=CONTRIBUTOR_ROLE)
def handle_added_contributor_role(event):
    """Handle added contributor role"""
    restrictions = IContributorRestrictions(event.object.__parent__, None)
    if restrictions is not None:
        restrictions.set_restrictions(event.principal_id, interface=IContributorRestrictions)


@subscriber(IRevokedRoleEvent, role_selector=CONTRIBUTOR_ROLE)
def handle_revoked_contributor_role(event):
    """Handle revoked contributor role"""
    restrictions = IContributorRestrictions(event.object.__parent__, None)
    if restrictions is not None:
        restrictions.drop_restrictions(event.principal_id)


#
# Manager restrictions
#

@adapter_config(required=IBaseSharedTool,
                provides=IManagerRestrictions)
def shared_tool_manager_restrictions(context):
    """Shared tool manager restrictions"""
    return get_annotation_adapter(context, MANAGERS_RESTRICTIONS_KEY,
                                  ISharedToolRestrictions)


@factory_config(IManagerWorkflowRestrictions)
class ManagerWorkflowRestrictions(Persistent):
    """Manager workflow restrictions"""

    weight = 10

    show_workflow_warning = FieldProperty(IManagerWorkflowRestrictions['show_workflow_warning'])
    restricted_contents = FieldProperty(IManagerWorkflowRestrictions['restricted_contents'])
    owners = FieldProperty(IManagerWorkflowRestrictions['owners'])

    def can_access(self, context, permission, request):
        """Access checker"""
        if not self.restricted_contents:
            return True
        roles = IWfSharedContentRoles(context, None)
        if roles is None:
            return False
        return bool(roles.owner & set(self.owners or ()))


@adapter_config(name='workflow',
                required=IManagerRestrictions,
                provides=IRestrictionInfo)
@adapter_config(required=IManagerRestrictions,
                provides=IManagerWorkflowRestrictions)
def manager_workflow_restrictions_adapter(context):
    """Manager workflow restrictions adapter"""
    return get_annotation_adapter(context, MANAGER_WORKFLOW_RESTRICTIONS_KEY,
                                  IManagerWorkflowRestrictions)


@adapter_config(required=ISharedContent,
                provides=IManagerRestrictions)
@adapter_config(required=IWfSharedContent,
                provides=IManagerRestrictions)
def shared_content_manager_restrictions(context):
    """Shared content manager restrictions"""
    tool = get_parent(context, IBaseSharedTool)
    if tool is not None:
        return IManagerRestrictions(tool)
    return None


@subscriber(IGrantedRoleEvent, role_selector=MANAGER_ROLE)
def handle_added_manager_role(event):
    """Handle added manager role"""
    restrictions = IManagerRestrictions(event.object.__parent__, None)
    if restrictions is not None:
        restrictions.set_restrictions(event.principal_id, interface=IManagerRestrictions)


@subscriber(IRevokedRoleEvent, role_selector=MANAGER_ROLE)
def handle_revoked_manager_role(event):
    """Handle revoked manager role"""
    restrictions = IManagerRestrictions(event.object.__parent__, None)
    if restrictions is not None:
        restrictions.drop_restrictions(event.principal_id)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.association.zmi module

This module provides base associations management components.
"""

from zope.interface import Interface, implementer

from pyams_content.component.association import IAssociationContainer, \
    IAssociationContainerTarget, IAssociationItem
from pyams_content.component.association.interfaces import IAssociationInfo
from pyams_content.component.association.zmi.interfaces import IAssociationItemAddForm, \
    IAssociationItemEditForm, IAssociationsTable
from pyams_content.component.paragraph.zmi import get_json_paragraph_toolbar_refresh_event
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_zmi.helper.event import get_json_table_row_add_callback, \
    get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, IObjectHint, IObjectIcon, IObjectLabel
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.table import TableElementEditor
from pyams_zmi.utils import get_object_hint, get_object_label


__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=(IAssociationContainer, IAdminLayer, Interface),
                provides=IObjectLabel)
def association_container_label(context, request, view):  # pylint: disable=unused-argument
    """Association container label getter"""
    target = get_parent(context, IAssociationContainerTarget)
    return get_object_label(target, request, view)


class AssociationItemAddMenuMixin:
    """Link add menu mixin class"""

    modal_target = True

    def get_href(self):
        """Menu URL target getter"""
        container = IAssociationContainer(self.context)
        return absolute_url(container, self.request, self.href)


@implementer(IAssociationItemAddForm)
class AssociationItemAddFormMixin:
    """Association item add form mixin class"""

    def add(self, obj):
        """Add file to container"""
        IAssociationContainer(self.context).append(obj)


@adapter_config(required=(IAssociationContainer, IAdminLayer, IAssociationItemAddForm),
                provides=IAJAXFormRenderer)
class AssociationItemAddFormRenderer(ContextRequestViewAdapter):
    """Association item add form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        result = {
            'status': 'success',
            'callbacks': [
                get_json_table_row_add_callback(self.context, self.request,
                                                IAssociationsTable, changes)
            ]
        }
        event = get_json_paragraph_toolbar_refresh_event(changes, self.request)
        if event is not None:
            result.setdefault('callbacks', []).append(event)
        return result


@adapter_config(required=(IAssociationItem, IAdminLayer, Interface),
                provides=IObjectLabel)
def association_item_label(context, request, view):  # pylint: disable=unused-argument
    """Association item label getter"""
    return IAssociationInfo(context).user_title


@adapter_config(required=(IAssociationItem, IAdminLayer, Interface),
                provides=IObjectIcon)
def association_item_icon(context, request, view):
    """Association item icon getter"""
    return f'fa-fw {context.icon_class}'


@adapter_config(required=(IAssociationItem, IAdminLayer, Interface),
                provides=IObjectHint)
def association_item_hint(context, request, view):
    """Association item hint getter"""
    return request.localizer.translate(context.icon_hint)


@adapter_config(required=(IAssociationItem, IAdminLayer, Interface),
                provides=ITableElementEditor)
class AssociationItemTableElementEditor(TableElementEditor):
    """Association item table element editor"""


@adapter_config(required=(IAssociationItem, IAdminLayer, IAssociationItemEditForm),
                provides=IFormTitle)
def association_item_edit_form_title(context, request, view):
    """Association item properties edit form title getter"""
    translate = request.localizer.translate
    parent = get_parent(context, IAssociationContainerTarget)
    hint = get_object_hint(parent, request, view)
    label = get_object_label(parent, request, view)
    parent_label = translate(_("{}: {}")).format(hint, label) if hint else label
    label = translate(_("{}: {}")).format(get_object_hint(context, request, view),
                                          get_object_label(context, request, view))
    return f'<small>{parent_label}</small><br />{label}'


@adapter_config(required=(IAssociationItem, IAdminLayer, IAssociationItemEditForm),
                provides=IAJAXFormRenderer)
class AssociationItemEditFormRenderer(ContextRequestViewAdapter):
    """Link edit form AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        parent = get_parent(self.context, IAssociationContainer)
        return {
            'callbacks': [
                get_json_table_row_refresh_callback(parent, self.request,
                                                    IAssociationsTable, self.context)
            ]
        }



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.links.interfaces module

"""

from zope.interface import Attribute, Interface
from zope.schema import Bool, Choice, InterfaceField, TextLine, URI

from pyams_content.component.association.interfaces import IAssociationContainerTarget, \
    IAssociationItem
from pyams_content.reference.pictogram.interfaces import SELECTED_PICTOGRAM_VOCABULARY
from pyams_i18n.interfaces import BASE_LANGUAGES_VOCABULARY_NAME
from pyams_i18n.schema import I18nTextField, I18nTextLineField
from pyams_sequence.interfaces import IInternalReference
from pyams_utils.schema import MailAddressField


__docformat__ = 'restructuredtext'

from pyams_content import _


CONTENT_LINKS_VOCABULARY = 'pyams_content.links'


class IBaseLink(IAssociationItem):
    """Base link interface"""

    title = I18nTextLineField(title=_("Alternate title"),
                              description=_("Link title, as shown in front-office"),
                              required=False)

    description = I18nTextField(title=_("Description"),
                                description=_(
                                    "Link description displayed by front-office template"),
                                required=False)

    pictogram_name = Choice(title=_("Pictogram"),
                            description=_("Name of the pictogram associated with this link"),
                            required=False,
                            vocabulary=SELECTED_PICTOGRAM_VOCABULARY)

    pictogram = Attribute("Selected pictogram object")

    def get_editor_url(self):
        """Get URL for use in HTML editor"""


INTERNAL_LINK_ICON_CLASS = 'fas fa-sign-in-alt fa-rotate-270'
INTERNAL_LINK_ICON_HINT = _("Internal link")


class IInternalLink(IBaseLink, IInternalReference):
    """Internal link interface"""

    force_canonical_url = Bool(title=_("Force canonical URL?"),
                               description=_("By default, internal links use a \"relative\" URL, "
                                             "which tries to display link target in the current "
                                             "context; by using a canonical URL, you can display "
                                             "target in it's attachment context (if defined)"),
                               required=False,
                               default=False)


#
# Custom internal link properties support
# These interfaces are used to be able to add custom properties to an internal link
# when it's target is of a given content type
#

class IInternalLinkCustomInfoTarget(Interface):
    """Internal link target info

    This optional interface can be supported be any content to be able to provide any
    additional information to link properties
    """

    internal_link_marker_interface = InterfaceField(title=_("Marker interface provided by links "
                                                            "directed to contents supporting "
                                                            "this interface"))


class ICustomInternalLinkTarget(Interface):
    """Base interface for custom internal link target"""


class IInternalLinkCustomInfo(Interface):
    """Base interface for custom link properties"""

    properties_interface = InterfaceField(title=_("Info properties interface"))

    def get_url_params(self):
        """Get custom params to generate link URL"""


EXTERNAL_LINK_ICON_CLASS = 'fas fa-link'
EXTERNAL_LINK_ICON_HINT = _("External link")


class IExternalLink(IBaseLink):
    """External link interface"""

    url = URI(title=_("Target URL"),
              description=_("URL used to access external resource"),
              required=True)

    language = Choice(title=_("Language"),
                      description=_("Language used in this remote resource"),
                      vocabulary=BASE_LANGUAGES_VOCABULARY_NAME,
                      required=False)


MAILTO_LINK_ICON_CLASS = 'fas fa-envelope'
MAILTO_LINK_ICON_HINT = _("Mailto link")


class IMailtoLink(IBaseLink):
    """Mailto link interface"""

    address = MailAddressField(title=_("Target address"),
                               description=_("Target email address"),
                               required=True)

    address_name = TextLine(title=_("Address name"),
                            description=_("Address as displayed in address book"),
                            required=True)


class ILinkContainerTarget(IAssociationContainerTarget):
    """Links container marker interface"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.site.manager module

"""

from pyramid.events import subscriber
from zope.container.ordered import OrderedContainer
from zope.interface import implementer
from zope.lifecycleevent import IObjectAddedEvent, IObjectMovedEvent, IObjectRemovedEvent
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.component.illustration import IIllustrationTarget, ILinkIllustrationTarget
from pyams_content.feature.preview.interfaces import IPreviewTarget
from pyams_content.interfaces import MANAGE_SITE_PERMISSION
from pyams_content.reference.pictogram.interfaces import IPictogramManagerTarget
from pyams_content.shared.common import ISharedContent
from pyams_content.shared.common.manager import BaseSharedTool
from pyams_content.shared.common.types import TypedSharedToolMixin
from pyams_content.shared.site.container import SiteContainerMixin
from pyams_content.shared.site.interfaces import ISiteManager, PYAMS_SITES_VOCABULARY, SITE_TOPIC_CONTENT_TYPE
from pyams_i18n.interfaces import II18n
from pyams_layer.skin import UserSkinnableContentMixin
from pyams_portal.interfaces import IPortalContext, IPortalFooterContext, IPortalHeaderContext
from pyams_security.interfaces import IDefaultProtectionPolicy, IViewContextPermissionChecker
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.factory import factory_config, get_object_factory
from pyams_utils.registry import get_utilities_for
from pyams_utils.request import query_request, check_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config

__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(ISiteManager)
@implementer(IDefaultProtectionPolicy, IPictogramManagerTarget,
             IIllustrationTarget, ILinkIllustrationTarget,
             IPortalContext, IPortalHeaderContext, IPortalFooterContext, IPreviewTarget)
class SiteManager(SiteContainerMixin, OrderedContainer, TypedSharedToolMixin, BaseSharedTool,
                  UserSkinnableContentMixin):
    """Site manager persistent class"""

    header = FieldProperty(ISiteManager['header'])
    description = FieldProperty(ISiteManager['description'])
    notepad = FieldProperty(ISiteManager['notepad'])
    navigation_mode = FieldProperty(ISiteManager['navigation_mode'])

    sequence_name = ''  # use default sequence generator
    sequence_prefix = ''

    content_name = _("Site manager")

    shared_content_type = SITE_TOPIC_CONTENT_TYPE

    @property
    def shared_content_factory(self):
        return get_object_factory(ISharedContent, name=self.shared_content_type)

    def is_deletable(self):
        for element in self.values():
            if not element.is_deletable():
                return False
        return True


@subscriber(IObjectAddedEvent, context_selector=ISiteManager)
def handle_added_site_manager(event: IObjectAddedEvent):
    """Register site manager when added"""
    site = get_parent(event.newParent, ISiteRoot)
    registry = site.getSiteManager()
    if registry is not None:
        registry.registerUtility(event.object, ISiteManager, name=event.newName)


@subscriber(IObjectMovedEvent, context_selector=ISiteManager)
def handle_moved_site_manager(event: IObjectMovedEvent):
    """Update site manager registration when renamed"""
    if IObjectRemovedEvent.providedBy(event):
        return
    request = check_request()
    registry = request.root.getSiteManager()
    if registry is not None:
        old_name = event.oldName
        new_name = event.newName
        if old_name == new_name:
            return
        registry.unregisterUtility(event.object, ISiteManager, name=old_name)
        if new_name:
            registry.registerUtility(event.object, ISiteManager, name=new_name)


@subscriber(IObjectRemovedEvent, context_selector=ISiteManager)
def handle_deleted_site_manager(event: IObjectRemovedEvent):
    """Un-register site manager when deleted"""
    site = get_parent(event.oldParent, ISiteRoot)
    registry = site.getSiteManager()
    if registry is not None:
        registry.unregisterUtility(event.object, ISiteManager, name=event.oldName)


@adapter_config(required=ISiteManager,
                provides=IViewContextPermissionChecker)
class SiteManagerPermissionChecker(ContextAdapter):
    """Site manager edit permission checker"""

    edit_permission = MANAGE_SITE_PERMISSION


@vocabulary_config(name=PYAMS_SITES_VOCABULARY)
class SiteManagerVocabulary(SimpleVocabulary):
    """Site manager vocabulary"""

    interface = ISiteManager

    def __init__(self, context=None):
        request = query_request()
        super().__init__([
            SimpleTerm(v, title=II18n(t).query_attribute('title', request=request))
            for v, t in get_utilities_for(self.interface)
        ])



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.workflow.interfaces module

This module defines custom workflow-related interfaces.
"""

from pyams_scheduler.interfaces import ITask
from pyams_workflow.interfaces import IWorkflow


__docformat__ = 'restructuredtext'


class IContentWorkflow(IWorkflow):
    """PyAMS default content workflow marker interface"""


class IBasicWorkflow(IWorkflow):
    """PyAMS basic workflow marker interface"""


class IWorkflowManagementTask(ITask):
    """Workflow management task marker interface

    This interface is used to mark scheduler tasks (see PyAMS_scheduler) which are
    used by some workflows which can provide a "future" publication date, and which
    relies on task scheduler to do the actual publication.
    """



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_shared.site.link module

"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import Interface, implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.illustration import ILinkIllustrationTarget
from pyams_content.component.links import ExternalLink, InternalLink
from pyams_content.feature.navigation.interfaces import IDynamicMenu
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_content.shared.site.interfaces import IExternalSiteLink, IInternalSiteLink, \
    ISiteElementNavigation, ISiteLink
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_sequence.reference import InternalReferenceMixin, get_reference_target
from pyams_utils.adapter import ContextAdapter, ContextRequestAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import MISSING_INFO
from pyams_utils.zodb import volatile_property
from pyams_workflow.interfaces import IWorkflow, IWorkflowPublicationInfo, IWorkflowState, \
    IWorkflowVersion, IWorkflowVersions
from pyams_zmi.interfaces import IObjectLabel
from pyams_zmi.utils import get_object_label

__docformat__ = 'restructuredtext'

from pyams_content import _


@implementer(ISiteLink, ILinkIllustrationTarget)
class SiteLink(Persistent, Contained):
    """Site link persistent class"""

    navigation_title = FieldProperty(ISiteLink['navigation_title'])
    show_header = FieldProperty(ISiteLink['show_header'])
    navigation_header = FieldProperty(ISiteLink['navigation_header'])
    visible = FieldProperty(ISiteLink['visible'])

    @staticmethod
    def is_deletable():
        """Link deletion checker"""
        return True


@adapter_config(required=ISiteLink,
                provides=IViewContextPermissionChecker)
class SiteLinkPermissionCheck(ContextAdapter):
    """Site link permission checker"""

    edit_permission = MANAGE_CONTENT_PERMISSION


#
# Internal content link
#

@factory_config(IInternalSiteLink)
class InternalSiteLink(InternalReferenceMixin, SiteLink):
    """Internal site link persistent class

    An 'internal content link' is a link to another content, which may be stored anywhere
    (same site, another site or in any shared tool).
    """

    _reference = FieldProperty(IInternalSiteLink['reference'])
    force_canonical_url = FieldProperty(IInternalSiteLink['force_canonical_url'])

    content_name = _("Internal link")

    @volatile_property
    def target(self):
        """Link target getter"""
        target = get_reference_target(self.reference)
        if IWorkflowVersion.providedBy(target):
            workflow = IWorkflow(target, None)
            if workflow is not None:
                versions = IWorkflowVersions(target).get_versions(workflow.visible_states,
                                                                  sort=True)
                if not versions:
                    versions = IWorkflowVersions(target).get_last_versions()
                if versions:
                    target = versions[-1]
        return target


@adapter_config(required=IInternalSiteLink,
                provides=IDynamicMenu)
def internal_site_link_dynamic_menu(context):
    """Internal site link dynamic menu factory"""
    target = context.get_target()
    if target is not None:
        result = InternalLink()
        result.title = context.navigation_title.copy() if context.navigation_title else {}
        result.reference = context.reference
        result.force_canonical_url = context.force_canonical_url
        return result
    return None


@adapter_config(required=(IInternalSiteLink, IPyAMSLayer),
                provides=ISiteElementNavigation)
class InternalSiteLinkNavigationAdapter(ContextRequestAdapter):
    """Internal site link navigation adapter"""

    @property
    def visible(self):
        """Navigation link visibility getter"""
        if not self.context.visible:
            return False
        target = self.context.target
        return (target is not None) and IWorkflowPublicationInfo(target).is_visible(self.request)


@adapter_config(required=(IInternalSiteLink, IPyAMSLayer, Interface),
                provides=IObjectLabel)
def internal_site_link_label(context, request, view):
    """Internal site link label"""
    label = II18n(context).query_attribute('navigation_title', request=request)
    if not label:
        target = context.get_target()
        if target is not None:
            label = get_object_label(target, request, view)
    return label or MISSING_INFO


@adapter_config(required=IInternalSiteLink,
                provides=IWorkflow)
def internal_site_link_workflow_info(context):
    """Internal site link workflow info"""
    target = context.get_target()
    if target is not None:
        return IWorkflow(target, None)
    return None


@adapter_config(required=IInternalSiteLink,
                provides=IWorkflowState)
def internal_content_link_state_info(context):
    """Internal content link workflow state info"""
    target = context.get_target()
    if target is not None:
        return IWorkflowState(target, None)
    return None


@adapter_config(required=IInternalSiteLink,
                provides=IWorkflowPublicationInfo)
def internal_site_link_publication_info(context):
    """Internal site link publication info"""
    target = context.get_target()
    if target is not None:
        return IWorkflowPublicationInfo(target, None)
    return None


#
# External content link
#

@factory_config(IExternalSiteLink)
class ExternalSiteLink(SiteLink):
    """External site link persistent class"""

    url = FieldProperty(IExternalSiteLink['url'])

    content_name = _("External link")


@adapter_config(required=IExternalSiteLink,
                provides=IDynamicMenu)
def external_site_link_dynamic_menu_factory(context):
    """External content link dynamic menu factory"""
    result = ExternalLink()
    result.title = context.navigation_title.copy() if context.navigation_title else {}
    result.url = context.url
    return result


@adapter_config(required=(IExternalSiteLink, IPyAMSLayer),
                provides=ISiteElementNavigation)
class ExternalSiteLinkNavigationAdapter(ContextRequestAdapter):
    """External site link navigation adapter"""

    @property
    def visible(self):
        """Navigation link visibility getter"""
        return self.context.visible


@adapter_config(required=(IExternalSiteLink, IPyAMSLayer, Interface),
                provides=IObjectLabel)
def external_site_link_label(context, request, view):
    """External site link label"""
    label = II18n(context).query_attribute('navigation_title', request=request)
    return label or context.url



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.alert.zmi module

This module defines components used for alerts management interface.
"""

from zope.interface import Interface

from pyams_content.interfaces import CREATE_CONTENT_PERMISSION
from pyams_content.shared.alert import IAlertManager, IWfAlert
from pyams_content.shared.common.zmi import SharedContentAddForm, SharedContentPropertiesEditForm
from pyams_content.zmi.interfaces import IDashboardColumn, IDashboardContentType
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IGroup
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IFormLayer, IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_utils.adapter import adapter_config
from pyams_zmi.form import FormGroupSwitcher
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContentManagementMenu, IMenuHeader

__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=(IWfAlert, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentType)
def wf_alert_content_type(context, request, column):
    """Alert content type"""
    alert_type = context.get_alert_type()
    if alert_type is not None:
        i18n = II18n(alert_type)
        return i18n.query_attributes_in_order(('dashboard_label', 'label'),
                                              request=request)


@adapter_config(required=(IWfAlert, IAdminLayer, Interface, IContentManagementMenu),
                provides=IMenuHeader)
def alert_management_menu_header(context, request, view, manager):
    """Alert management menu header"""
    return request.localizer.translate(_("Alert management"))


@ajax_form_config(name='add-shared-content.html',
                  context=IAlertManager, layer=IFormLayer,
                  permission=CREATE_CONTENT_PERMISSION)
class AlertAddForm(SharedContentAddForm):
    """Alert add form"""

    fields = Fields(IWfAlert).select('title', 'alert_type', 'notepad')


@ajax_form_config(name='properties.html',
                  context=IWfAlert, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class AlertPropertiesEditForm(SharedContentPropertiesEditForm):
    """Alert properties edit form"""

    interface = IWfAlert


@adapter_config(name='alert-properties',
                required=(IWfAlert, IAdminLayer, AlertPropertiesEditForm),
                provides=IGroup)
class AlertPropertiesGroup(FormGroupSwitcher):
    """Alert properties group"""

    legend = _("Alert settings")

    fields = Fields(IWfAlert).select('alert_type', 'body', 'reference', 'external_url',
                                     'references', 'maximum_interval')



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery.skin module

"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.gallery.interfaces import IGalleryParagraph
from pyams_content.component.gallery.skin.interfaces import IGalleryCarouselRendererSettings, \
    IGalleryDefaultRendererSettings, IGalleryRandomImageRendererSettings
from pyams_content.feature.renderer import BaseContentRenderer, IContentRenderer
from pyams_portal.interfaces import DEFAULT_RENDERER_NAME
from pyams_layer.interfaces import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.list import random_iter


__docformat__ = 'restructuredtext'

from pyams_content import _


class BaseGalleryRenderer(BaseContentRenderer):
    """Base gallery renderer"""


#
# Default gallery renderer
#

@factory_config(IGalleryDefaultRendererSettings)
class GalleryDefaultRendererSettings(Persistent, Contained):
    """Gallery default renderer settings"""

    thumb_selection = FieldProperty(
        IGalleryDefaultRendererSettings['thumb_selection'])

    def get_css_cols(self):
        """Get CSS cols matching current selections"""
        return ' '.join([
            f"col-{'{}-'.format(device) if device != 'xs' else ''}{selection.cols}"
            for device, selection in self.thumb_selection.items()
        ])


@adapter_config(name=DEFAULT_RENDERER_NAME,
                required=(IGalleryParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/gallery-default.pt', layer=IPyAMSLayer)
class GalleryParagraphDefaultRenderer(BaseGalleryRenderer):
    """Gallery paragraph default renderer"""

    label = _("Grid gallery renderer (default)")

    settings_interface = IGalleryDefaultRendererSettings
    weight = 10


#
# Carousel gallery renderer
#

@factory_config(IGalleryCarouselRendererSettings)
class GalleryCarouselRendererSettings(Persistent, Contained):
    """Gallery carousel renderer settings"""

    thumb_selection = FieldProperty(
        IGalleryCarouselRendererSettings['thumb_selection'])


@adapter_config(name='carousel',
                required=(IGalleryParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/gallery-carousel.pt', layer=IPyAMSLayer)
class GalleryParagraphCarouselRenderer(BaseGalleryRenderer):
    """Gallery paragraph carousel renderer"""

    label = _("Carousel gallery renderer")

    settings_interface = IGalleryCarouselRendererSettings
    weight = 20


#
# Random image gallery renderer
#

@factory_config(IGalleryRandomImageRendererSettings)
class GalleryRandomImageRendererSettings(Persistent, Contained):
    """Gallery random image renderer settings"""

    thumb_selection = FieldProperty(
        IGalleryRandomImageRendererSettings['thumb_selection'])


@adapter_config(name='random',
                required=(IGalleryParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/gallery-random.pt', layer=IPyAMSLayer)
class GalleryRandomImageRenderer(BaseGalleryRenderer):
    """Gallery paragraph random image renderer"""

    label = _("Random image renderer")

    settings_interface = IGalleryRandomImageRendererSettings
    weight = 30

    def get_media(self):
        """Visible media getter"""
        return next(random_iter(self.context.get_visible_medias()))



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.extfile.zmi.widget module

This module provides a custom widget used for external files title field.
"""

__docformat__ = 'restructuredtext'

from zope.interface import implementer_only

from pyams_content.component.extfile import IExtFileManagerInfo
from pyams_form.browser.text import TextWidget
from pyams_form.interfaces import DISPLAY_MODE, INPUT_MODE
from pyams_form.interfaces.widget import ITextWidget
from pyams_form.template import widget_template_config
from pyams_form.widget import FieldWidget
from pyams_i18n.interfaces import II18n
from pyams_i18n_views.interfaces import II18nWidget
from pyams_i18n_views.widget import I18nWidget
from pyams_layer.interfaces import IPyAMSLayer


class IExtFileTitleWidget(ITextWidget):
    """External file title widget marker interface"""


@widget_template_config(mode=INPUT_MODE,
                        template='templates/extfile-title-input.pt', layer=IPyAMSLayer)
@widget_template_config(mode=DISPLAY_MODE,
                        template='templates/extfile-title-display.pt', layer=IPyAMSLayer)
@implementer_only(IExtFileTitleWidget)
class ExtFileTitleWidget(TextWidget):
    """External file title widget"""

    @property
    def prefix(self):
        """Widget prefix getter"""
        info = IExtFileManagerInfo(self.request.root, None)
        if info is not None:
            lang = getattr(self, 'lang', None)
            if lang is not None:
                return (info.default_title_prefix or {}).get(lang, '') or ''
            return II18n(info).query_attribute('default_title_prefix', request=self.request)
        return ''


def ExtFileTitleFieldWidget(field, request):  # pylint: disable=invalid-name
    """External file title widget factory"""
    return FieldWidget(field, ExtFileTitleWidget(request))


#
# I18n external file title field widget
#

class II18nExtFileTitleWidget(II18nWidget):
    """I18n external file title field widget interface"""


class I18nExtFileTitleWidget(I18nWidget):
    """I18n external file title widget"""


def I18nExtFileTitleFieldWidget(field, request):  # pylint: disable=invalid-name
    """I18n external file title widget factory"""
    widget = I18nExtFileTitleWidget(request)
    widget.widget_factory = ExtFileTitleFieldWidget
    return FieldWidget(field, widget)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi.rename module

This module defines components which can be used to rename a site element.
"""

from pyramid.events import subscriber
from zope.container.interfaces import IContainer, IOrderedContainer
from zope.interface import Invalid
from zope.lifecycleevent import ObjectMovedEvent

from pyams_content.interfaces import MANAGE_SITE_PERMISSION
from pyams_content.shared.site.interfaces import IBaseSiteItem
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.traversing import get_parent
from pyams_utils.unicode import translate_string
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalEditForm
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.viewlet import IContextActionsDropdownMenu
from pyams_zmi.utils import get_object_label

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='rename-site-item.menu',
                context=IBaseSiteItem, layer=IAdminLayer,
                manager=IContextActionsDropdownMenu, weight=10,
                permission=MANAGE_SITE_PERMISSION)
class SiteItemRenameMenu(MenuItem):
    """Site item rename menu"""

    label = _("Change URL")
    icon_class = 'fas fa-pencil-alt'

    href = 'rename-site-item.html'
    modal_target = True


@ajax_form_config(name='rename-site-item.html',
                  context=IBaseSiteItem, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_PERMISSION)
class SiteItemRenameForm(AdminModalEditForm):
    """Site item rename form"""

    subtitle = _("Change item URL")
    legend = _("New item URL")

    fields = Fields(IBaseSiteItem).select('__name__')

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        name = self.widgets.get('__name__')
        if name is not None:
            name.required = True
            name.label = _("Item URL")
            name.description = _("URL part used to access this item")

    def apply_changes(self, data):
        data = data.get(self, data)
        content = self.get_content()
        parent = content.__parent__
        order = list(parent.keys())
        old_name = content.__name__
        new_name = data['__name__'] = translate_string(data.get('__name__', ''),
                                                       force_lower=True, spaces='-')
        changes = super().apply_changes(data)
        if changes:
            parent[new_name] = content
            del parent[old_name]
            self.request.registry.notify(ObjectMovedEvent(content, parent, old_name,
                                                          parent, new_name))
            if IOrderedContainer.providedBy(parent):
                # restore keys order
                order[order.index(old_name)] = new_name
                parent.updateOrder(order)
        return changes


@adapter_config(required=(IBaseSiteItem, IAdminLayer, SiteItemRenameForm),
                provides=IFormTitle)
def base_site_item_rename_form_title(context, request, form):
    """Base site item rename form title"""
    parent = get_parent(context, IContainer, allow_context=False)
    return TITLE_SPAN_BREAK.format(
        get_object_label(parent, request, form),
        get_object_label(context, request, form))


@subscriber(IDataExtractedEvent, form_selector=SiteItemRenameForm)
def handle_rename_form_data(event):
    """Handle form rename data"""
    data = event.data
    form = event.form
    if not data.get('__name__'):
        form.widgets.errors += (Invalid(_("Item URL can't be empty!")),)
    else:
        context = form.context
        new_name = translate_string(data.get('__name__', ''), force_lower=True, spaces='-')
        if (new_name in context.__parent__) and (new_name != context.__name__):
            form.widgets.errors += (Invalid(_("Selected name is already used!")),)


@adapter_config(required=(IBaseSiteItem, IAdminLayer, SiteItemRenameForm),
                provides=IAJAXFormRenderer)
class SiteItemRenameFormRenderer(ContextRequestViewAdapter):
    """Site item rename form renderer"""

    def render(self, changes):
        """Form renderer"""
        if changes is None:
            return None
        return {
            'status': 'redirect',
            'location': absolute_url(self.context, self.request, 'admin'),
            'smallbox': {
                'status': 'success',
                'message': self.request.localizer.translate(self.view.success_message)
            }
        }



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.alert module

This module defines alerts persistent classes.
"""

from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.thesaurus import ITagsTarget, IThemesTarget
from pyams_content.feature.review import IReviewTarget
from pyams_content.shared.alert.interfaces import ALERT_CONTENT_NAME, ALERT_CONTENT_TYPE, IAlert, IAlertManager, \
    IAlertTypesManager, IWfAlert
from pyams_content.shared.common import ISharedContent, IWfSharedContent, SharedContent, WfSharedContent
from pyams_sequence.reference import InternalReferenceMixin, get_reference_target
from pyams_utils.factory import factory_config
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent

__docformat__ = 'restructuredtext'


@factory_config(IWfAlert)
@factory_config(IWfSharedContent, name=ALERT_CONTENT_TYPE)
@implementer(ITagsTarget, IThemesTarget, IReviewTarget)
class WfAlert(WfSharedContent, InternalReferenceMixin):
    """Base alert"""

    content_type = ALERT_CONTENT_TYPE
    content_name = ALERT_CONTENT_NAME
    content_intf = IWfAlert
    content_view = False

    handle_content_url = False
    handle_header = False
    handle_description = False

    alert_type = FieldProperty(IWfAlert['alert_type'])
    body = FieldProperty(IWfAlert['body'])
    _reference = FieldProperty(IWfAlert['reference'])
    external_url = FieldProperty(IWfAlert['external_url'])
    references = FieldProperty(IWfAlert['references'])
    maximum_interval = FieldProperty(IWfAlert['maximum_interval'])

    def get_alert_type(self):
        """Alert type getter"""
        manager = get_parent(self, IAlertManager)
        types = IAlertTypesManager(manager, None)
        if types is not None:
            return types.get(self.alert_type)

    @property
    def reference(self):
        return self._reference

    @reference.setter
    def reference(self, value):
        self._reference = value
        del self.target

    def get_targets(self, state=None):
        request = check_request()
        return [
            get_reference_target(reference, state, request)
            for reference in (self.references or ())
        ]


@factory_config(IAlert)
@factory_config(ISharedContent, name=ALERT_CONTENT_TYPE)
class Alert(SharedContent):
    """Workflow managed alert class"""

    content_type = ALERT_CONTENT_TYPE
    content_name = ALERT_CONTENT_NAME
    content_view = False



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.security module

This module defines common security rules used by all shared tools.
"""

from zope.interface import implementer
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.shared.common import IWfSharedContent
from pyams_content.shared.common.interfaces import CONTENT_MANAGER_CONTRIBUTORS, CONTENT_MANAGER_ROLES, \
    IBaseSharedTool, ISharedToolRoles, IWfSharedContentRoles, SHARED_CONTENT_ROLES
from pyams_security.interfaces import IRolesPolicy
from pyams_security.property import RolePrincipalsFieldProperty
from pyams_security.security import ProtectedObjectRoles
from pyams_security.utility import get_principal
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.vocabulary import vocabulary_config

__docformat__ = 'restructuredtext'


@implementer(ISharedToolRoles)
class SharedToolRoles(ProtectedObjectRoles):
    """Shared tool roles"""

    webmasters = RolePrincipalsFieldProperty(ISharedToolRoles['webmasters'])
    pilots = RolePrincipalsFieldProperty(ISharedToolRoles['pilots'])
    managers = RolePrincipalsFieldProperty(ISharedToolRoles['managers'])
    contributors = RolePrincipalsFieldProperty(ISharedToolRoles['contributors'])
    designers = RolePrincipalsFieldProperty(ISharedToolRoles['designers'])


@adapter_config(required=IBaseSharedTool,
                provides=ISharedToolRoles)
def shared_tool_roles_adapter(context):
    """Shared tool roles adapter"""
    return SharedToolRoles(context)


@adapter_config(name=CONTENT_MANAGER_ROLES,
                required=IBaseSharedTool,
                provides=IRolesPolicy)
class SharedToolRolesPolicy(ContextAdapter):
    """Shared tool roles policy"""

    roles_interface = ISharedToolRoles
    weight = 10


@implementer(IWfSharedContentRoles)
class WfSharedContentRoles(ProtectedObjectRoles):
    """Shared content roles"""

    owner = RolePrincipalsFieldProperty(IWfSharedContentRoles['owner'])
    managers = RolePrincipalsFieldProperty(IWfSharedContentRoles['managers'])
    contributors = RolePrincipalsFieldProperty(IWfSharedContentRoles['contributors'])
    designers = RolePrincipalsFieldProperty(IWfSharedContentRoles['designers'])
    readers = RolePrincipalsFieldProperty(IWfSharedContentRoles['readers'])
    guests = RolePrincipalsFieldProperty(IWfSharedContentRoles['guests'])


@adapter_config(required=IWfSharedContent,
                provides=IWfSharedContentRoles)
def shared_content_roles_adapter(context):
    """Shared content roles adapter"""
    return WfSharedContentRoles(context)


@adapter_config(name=SHARED_CONTENT_ROLES,
                required=IWfSharedContent,
                provides=IRolesPolicy)
class SharedContentRolesPolicy(ContextAdapter):
    """Shared content roles policy"""

    roles_interface = IWfSharedContentRoles
    weight = 10


@vocabulary_config(name=CONTENT_MANAGER_CONTRIBUTORS)
class SharedToolContributorsVocabulary(SimpleVocabulary):
    """Shared tool contributors vocabulary"""

    def __init__(self, context):
        roles = ISharedToolRoles(context, None)
        terms = []
        for principal_id in roles.contributors or set():
            principal = get_principal(None, principal_id)
            if principal is not None:
                terms.append(SimpleTerm(principal.id, title=principal.title))
        super().__init__(terms)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.glossary.zmi module

This module defines components used to handle glossary updater task.
"""

from pyams_content.component.thesaurus import ITagsManagerTarget
from pyams_content.component.thesaurus.zmi.manager import TagsManagerGlossaryGroup
from pyams_content.feature.glossary.interfaces import IGlossaryUpdaterTask
from pyams_content.feature.glossary.task import GlossaryUpdaterTask
from pyams_form.ajax import ajax_form_config
from pyams_layer.interfaces import IPyAMSLayer
from pyams_scheduler.interfaces import ITaskContainer, MANAGE_TASKS_PERMISSION
from pyams_scheduler.task.zmi import BaseTaskAddForm, BaseTaskEditForm
from pyams_scheduler.zmi import TaskContainerTable
from pyams_skin.interfaces.viewlet import IHelpViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_skin.viewlet.menu import MenuItem
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager


__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='tags-glossary.help',
                context=ITagsManagerTarget, layer=IAdminLayer, view=TagsManagerGlossaryGroup,
                manager=IHelpViewletManager, weight=1)
class TagsGlossaryHeader(AlertMessage):
    """Tags manager glossary help message"""

    status = 'info'

    _message = _("To use glossary features, you must define a glossary updater task "
                 "into tasks scheduler utility!")


@viewlet_config(name='add-glossary-updater-task.menu',
                context=ITaskContainer, layer=IAdminLayer, view=TaskContainerTable,
                manager=IContextAddingsViewletManager, weight=210,
                permission=MANAGE_TASKS_PERMISSION)
class GlossaryUpdaterTaskAddMenu(MenuItem):
    """Glossary updater task add menu"""

    label = _("Add glossary updater task...")
    href = 'add-glossary-updater-task.html'
    modal_target = True


@ajax_form_config(name='add-glossary-updater-task.html',
                  context=ITaskContainer, layer=IPyAMSLayer,
                  permission=MANAGE_TASKS_PERMISSION)
class GlossaryUpdaterTaskAddForm(BaseTaskAddForm):
    """Glossary updater task add form"""

    content_factory = IGlossaryUpdaterTask
    content_label = GlossaryUpdaterTask.label


@ajax_form_config(name='properties.html',
                  context=IGlossaryUpdaterTask, layer=IPyAMSLayer,
                  permission=MANAGE_TASKS_PERMISSION)
class GlossaryUpdaterTaskEditForm(BaseTaskEditForm):
    """Glossary updater task edit form"""
